-- Fix fr mpuls/issue1922

-- Die Triggerfunktionen aller Repeatgroups werden erweitert, um
-- zustzlich zur Aktualisierung der case_structure Tabelle auch noch
-- set_lastaccess_time aufzurufen.
-- set_lastaccess_time muss auerdem auch fr Updates aufgerufen werden,
-- weshalb hier auch noch UPDATE trigger definiert werden.

BEGIN;

-- Continue as db-owner from here
SET ROLE :adm_ka_owner;


INSERT INTO db_history (beschreibung, zeitpunkt)
       VALUES ('2.0.2/01_rg_trigger.sql', current_timestamp);


--
-- rg_ansprechpartner_tbl
--

CREATE OR REPLACE FUNCTION rg_ansprechpartner_tbl_insert_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid NEW.master_id;
    UPDATE case_structure
        SET modified = FALSE, cache = compute_case_structure(master_id)
        WHERE master_id = mid;
    PERFORM set_lastaccess_time(mid);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION rg_ansprechpartner_tbl_update_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid NEW.master_id;
    PERFORM set_lastaccess_time(mid);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER rg_ansprechpartner_tbl_update_trigger AFTER UPDATE ON rg_ansprechpartner_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_ansprechpartner_tbl_update_func();

CREATE OR REPLACE FUNCTION rg_ansprechpartner_tbl_delete_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid OLD.master_id;
    UPDATE case_structure
        SET modified = FALSE, cache = compute_case_structure(master_id)
        WHERE master_id = mid;
    PERFORM set_lastaccess_time(mid);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;


--
-- rg_kontaktdaten_tbl
--

CREATE OR REPLACE FUNCTION rg_kontaktdaten_tbl_insert_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid NEW.master_id;
    UPDATE case_structure
        SET modified = FALSE, cache = compute_case_structure(master_id)
        WHERE master_id = mid;
    PERFORM set_lastaccess_time(mid);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;


CREATE OR REPLACE FUNCTION rg_kontaktdaten_tbl_update_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid NEW.master_id;
    PERFORM set_lastaccess_time(mid);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER rg_kontaktdaten_tbl_update_trigger AFTER UPDATE ON rg_kontaktdaten_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_kontaktdaten_tbl_update_func();

CREATE OR REPLACE FUNCTION rg_kontaktdaten_tbl_delete_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid OLD.master_id;
    UPDATE case_structure
        SET modified = FALSE, cache = compute_case_structure(master_id)
        WHERE master_id = mid;
    PERFORM set_lastaccess_time(mid);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;

--
-- rg_fehlzeiten_tbl
--

CREATE OR REPLACE FUNCTION rg_fehlzeiten_tbl_insert_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid NEW.master_id;
    UPDATE case_structure
        SET modified = FALSE, cache = compute_case_structure(master_id)
        WHERE master_id = mid;
    PERFORM set_lastaccess_time(mid);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION rg_fehlzeiten_tbl_update_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid NEW.master_id;
    PERFORM set_lastaccess_time(mid);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER rg_fehlzeiten_tbl_update_trigger AFTER UPDATE ON rg_fehlzeiten_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_fehlzeiten_tbl_update_func();

CREATE OR REPLACE FUNCTION rg_fehlzeiten_tbl_delete_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid OLD.master_id;
    UPDATE case_structure
        SET modified = FALSE, cache = compute_case_structure(master_id)
        WHERE master_id = mid;
    PERFORM set_lastaccess_time(mid);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;

--
-- rg_probleme_tbl
--

CREATE OR REPLACE FUNCTION rg_probleme_tbl_insert_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid NEW.master_id;
    UPDATE case_structure
        SET modified = FALSE, cache = compute_case_structure(master_id)
        WHERE master_id = mid;
    PERFORM set_lastaccess_time(mid);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION rg_probleme_tbl_update_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid NEW.master_id;
    PERFORM set_lastaccess_time(mid);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER rg_probleme_tbl_update_trigger AFTER UPDATE ON rg_probleme_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_probleme_tbl_update_func();

CREATE OR REPLACE FUNCTION rg_probleme_tbl_delete_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid OLD.master_id;
    UPDATE case_structure
        SET modified = FALSE, cache = compute_case_structure(master_id)
        WHERE master_id = mid;
    PERFORM set_lastaccess_time(mid);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;

--
-- rg_hilfsmassnahmen_tbl
--

CREATE OR REPLACE FUNCTION rg_hilfsmassnahmen_tbl_insert_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid NEW.master_id;
    UPDATE case_structure
        SET modified = FALSE, cache = compute_case_structure(master_id)
        WHERE master_id = mid;
    PERFORM set_lastaccess_time(mid);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION rg_hilfsmassnahmen_tbl_update_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid NEW.master_id;
    PERFORM set_lastaccess_time(mid);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER rg_hilfsmassnahmen_tbl_update_trigger AFTER UPDATE ON rg_hilfsmassnahmen_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_hilfsmassnahmen_tbl_update_func();

CREATE OR REPLACE FUNCTION rg_hilfsmassnahmen_tbl_delete_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid OLD.master_id;
    UPDATE case_structure
        SET modified = FALSE, cache = compute_case_structure(master_id)
        WHERE master_id = mid;
    PERFORM set_lastaccess_time(mid);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;

--
-- rg_vereinbarungen_tbl
--

CREATE OR REPLACE FUNCTION rg_vereinbarungen_tbl_insert_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid NEW.master_id;
    UPDATE case_structure
        SET modified = FALSE, cache = compute_case_structure(master_id)
        WHERE master_id = mid;
    PERFORM set_lastaccess_time(mid);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION rg_vereinbarungen_tbl_update_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid NEW.master_id;
    PERFORM set_lastaccess_time(mid);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER rg_vereinbarungen_tbl_update_trigger AFTER UPDATE ON rg_vereinbarungen_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_vereinbarungen_tbl_update_func();

CREATE OR REPLACE FUNCTION rg_vereinbarungen_tbl_delete_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid OLD.master_id;
    UPDATE case_structure
        SET modified = FALSE, cache = compute_case_structure(master_id)
        WHERE master_id = mid;
    PERFORM set_lastaccess_time(mid);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;

--
-- rg_elternarbeit_tbl
--

CREATE OR REPLACE FUNCTION rg_elternarbeit_tbl_insert_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid NEW.master_id;
    UPDATE case_structure
        SET modified = FALSE, cache = compute_case_structure(master_id)
        WHERE master_id = mid;
    PERFORM set_lastaccess_time(mid);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION rg_elternarbeit_tbl_update_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid NEW.master_id;
    PERFORM set_lastaccess_time(mid);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER rg_elternarbeit_tbl_update_trigger AFTER UPDATE ON rg_elternarbeit_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_elternarbeit_tbl_update_func();

CREATE OR REPLACE FUNCTION rg_elternarbeit_tbl_delete_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid OLD.master_id;
    UPDATE case_structure
        SET modified = FALSE, cache = compute_case_structure(master_id)
        WHERE master_id = mid;
    PERFORM set_lastaccess_time(mid);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;


--
--
--

COMMIT;

