BEGIN;

-- Change to use password length 12 letters. Needs to replace functions
-- mpuls_create_role, mpuls_reset_role, ka_create_role

INSERT INTO db_history (beschreibung, zeitpunkt)
VALUES ('2.0.4/01-pwgen-12.sql', current_timestamp);

CREATE OR REPLACE FUNCTION mpuls_create_role (
        username varchar(64),
        grouprole varchar(64), 
        agency varchar(52)
  ) RETURNS RECORD LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
  AS $$
  DECLARE 
    db_name  varchar(64);
    db_user  varchar(128);
    db_group varchar(64);
    pb_group varchar(64);
    pw       varchar(13);
    result   RECORD;
  BEGIN
    -- IF position(quote_literal(agency) in session_user) != 4
    IF position(agency in session_user) != 4
    THEN
      RAISE EXCEPTION 
          'Permission denied to create role %s in agency % by %.',
          quote_literal(username), quote_literal(agency), session_user;
    ELSE
      db_name := quote_ident('ka_' || agency || '_db');
      db_user := quote_ident('ka_' || agency || '_' || username);
      IF lower(grouprole) = 'admin' THEN
        db_group := quote_ident('ka_' || agency || '_admin');
      ELSIF lower(grouprole) = 'pb' THEN
        db_group := quote_ident('ka_' || agency || '_pb');
      ELSE
        db_group := quote_ident('ka_' || agency || '_cm');
      END IF;
      pw := mpuls_passgen(12);
      EXECUTE 'CREATE ROLE ' || db_user 
          || ' LOGIN NOSUPERUSER INHERIT NOCREATEDB NOCREATEROLE'
          || ' PASSWORD ' || quote_literal(pw);
      EXECUTE 'GRANT ' || db_group || ' TO ' || db_user;
      SELECT INTO result db_name::varchar(64), db_user::varchar(64), pw::varchar(13);
      RETURN result;
    END IF;

  END;
$$ ;
REVOKE ALL ON FUNCTION mpuls_create_role( varchar, varchar, varchar ) 
    FROM public;
GRANT EXECUTE ON FUNCTION mpuls_create_role( varchar, varchar, varchar ) 
    TO admin;

CREATE OR REPLACE FUNCTION mpuls_reset_password (
        name varchar(64),
        agency varchar(52)
  ) RETURNS text LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
  AS $$
  DECLARE 
    db_name  varchar(64);
    db_user  varchar(128);
    pw       varchar(13);
  BEGIN
    -- IF position(quote_literal(agency) in session_user) != 4
    IF position(agency in session_user) != 4
    THEN
      RAISE EXCEPTION 
          'Permission denied to reset password for role % in agency % by %.',
          quote_literal(name), quote_literal(agency), session_user;
    ELSE
      db_name := quote_ident('ka_' || agency || '_db');
      db_user := quote_ident('ka_' || agency || '_' || name);
      pw := mpuls_passgen(12);
      EXECUTE 'ALTER USER ' || db_user || ' WITH PASSWORD ' 
          || quote_literal(pw);
      RETURN pw::text;
    END IF;
    RETURN ''::text;
  END;
$$ ;
REVOKE ALL ON FUNCTION mpuls_reset_password( varchar, varchar ) FROM public;
GRANT EXECUTE ON FUNCTION mpuls_reset_password( varchar, varchar ) 
    TO admin;

CREATE OR REPLACE FUNCTION ka_create_role (
        agency          varchar(52),
        grouprole       varchar(64),
        loginname       varchar(64),
        firstname       varchar(64),
        surname         varchar(64),
        phone           varchar(64),
        room            varchar(64),
        branchoffice    varchar(64),
        activated       boolean,
        setpassword     boolean
    ) RETURNS varchar(13) 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    newrole    RECORD;
    groupid    integer;
    userid     integer;
BEGIN
    SELECT INTO newrole * FROM mpuls_create_role(loginname, grouprole, agency)
        AS (db_name varchar(64), db_user varchar(64), pw varchar(13));
    EXECUTE 'GRANT CONNECT ON DATABASE ' || newrole.db_name 
                || ' TO ' || newrole.db_user; 
    INSERT INTO ka_benutzer_tbl ( login, vorname, nachname,
        telefon, raumnummer, filiale,
        aktiviert, passwort_aendern, gid) 
        VALUES
        (newrole.db_user, firstname, surname,
        phone, room, branchoffice,
        activated, setpassword, NULL);
    SELECT INTO userid currval('ka_benutzer_tbl_id_seq');
    SELECT INTO groupid create_benutzergruppe_ds(surname ||', '|| firstname ||' ('||loginname||')');
    UPDATE ka_benutzer_tbl SET gid = groupid WHERE id = userid;
    INSERT INTO nm_benutzer_gruppe_tbl (benutzer_id, gruppe_id) VALUES (userid, groupid);

    RETURN newrole.pw::varchar(13);
END;
$$ ;

REVOKE ALL ON FUNCTION ka_create_role(varchar, varchar, varchar, 
    varchar, varchar, varchar, varchar, varchar, boolean, boolean) 
        FROM PUBLIC;
GRANT EXECUTE ON FUNCTION ka_create_role(varchar, varchar, varchar, 
    varchar, varchar, varchar, varchar, varchar, boolean, boolean) 
        TO GROUP :admin_group;


COMMIT;
