BEGIN;

-- 
-- Repeat Group fuer fehlzeiten
-- 

ALTER TABLE master_tbl DISABLE TRIGGER USER;

SET ROLE :db_adm;

INSERT INTO db_history (beschreibung, zeitpunkt)
       VALUES ('2.0/02_rg_fehlzeiten.sql', current_timestamp);


-- Create a new function to generate RG-entries.
-- This is the original function, without the DS IS check and exception
CREATE OR REPLACE FUNCTION create_rg_fehlzeiten_tbl(pid integer, nuuid char(36))
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid integer;
    mid integer;
BEGIN
    mid := pid;

    IF NOT isEditorOrStandin(mid) THEN
        -- RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_fehlzeiten_tbl (master_id) VALUES (pid);

    SELECT INTO rid currval('rg_fehlzeiten_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_fehlzeiten_tbl
        SET    uuid_id = nuuid
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_fehlzeiten_tbl', rid);
        
    END IF;

    RETURN rid;
END;
$$;


-- Uebernahme der alten Eintraege in eine Repeat Group
-- Fuer alle Eintrage in der master_tbl
CREATE OR REPLACE FUNCTION insert_rg() RETURNS VOID AS $$
DECLARE
  r RECORD;
  rgid INTEGER;
  rg_count INTEGER;
  uuid TEXT;
BEGIN
  FOR r in SELECT *
  FROM master_tbl LOOP
  
      -- Zunaechst alle RGs loeschen
      DELETE from rg_fehlzeiten_tbl WHERE master_id = r.id;

      IF r.fehlm_1 != '-1'
         OR r.fehlj_1 != '-1'
         OR r.fehltg_1 IS NOT NULL
         OR r.fehlutg_1 IS NOT NULL
         OR r.fehlst_1 IS NOT NULL
         OR r.fehlfust_1 IS NOT NULL THEN

         uuid = generate_uuidgen();
         rgid = create_rg_fehlzeiten_tbl(r.id, uuid);

         UPDATE rg_fehlzeiten_tbl set
              fehlzeiten_monat = r.fehlm_1,
              fehlzeiten_jahr = r.fehlj_1,
              fehlzeiten_anzahl_fehltage = r.fehltg_1,
              fehlzeiten_unentschuldigt_tage = r.fehlutg_1,
              fehlzeiten_anzahl_fehlstunden = r.fehlst_1,
              fehlzeiten_unentschuldigt_stunden = r.fehlfust_1
         WHERE id = rgid;    
      END IF;


      IF r.fehlm_2 != '-1'
         OR r.fehlj_2 != '-1'
         OR r.fehltg_2 IS NOT NULL
         OR r.fehlutg_2 IS NOT NULL
         OR r.fehlst_2 IS NOT NULL
         OR r.fehlfust_2 IS NOT NULL THEN

         uuid = generate_uuidgen();
         rgid = create_rg_fehlzeiten_tbl(r.id, uuid);

         UPDATE rg_fehlzeiten_tbl set
              fehlzeiten_monat = r.fehlm_2,
              fehlzeiten_jahr = r.fehlj_2,
              fehlzeiten_anzahl_fehltage = r.fehltg_2,
              fehlzeiten_unentschuldigt_tage = r.fehlutg_2,
              fehlzeiten_anzahl_fehlstunden = r.fehlst_2,
              fehlzeiten_unentschuldigt_stunden = r.fehlfust_2
         WHERE id = rgid;    
      END IF;


      IF r.fehlm_3 != '-1'
         OR r.fehlj_3 != '-1'
         OR r.fehltg_3 IS NOT NULL
         OR r.fehlutg_3 IS NOT NULL
         OR r.fehlst_3 IS NOT NULL
         OR r.fehlfust_3 IS NOT NULL THEN

         uuid = generate_uuidgen();
         rgid = create_rg_fehlzeiten_tbl(r.id, uuid);

         UPDATE rg_fehlzeiten_tbl set
              fehlzeiten_monat = r.fehlm_3,
              fehlzeiten_jahr = r.fehlj_3,
              fehlzeiten_anzahl_fehltage = r.fehltg_3,
              fehlzeiten_unentschuldigt_tage = r.fehlutg_3,
              fehlzeiten_anzahl_fehlstunden = r.fehlst_3,
              fehlzeiten_unentschuldigt_stunden = r.fehlfust_3
         WHERE id = rgid;    
      END IF;


      IF r.fehlm_4 != '-1'
         OR r.fehlj_4 != '-1'
         OR r.fehltg_4 IS NOT NULL
         OR r.fehlutg_4 IS NOT NULL
         OR r.fehlst_4 IS NOT NULL
         OR r.fehlfust_4 IS NOT NULL THEN

         uuid = generate_uuidgen();
         rgid = create_rg_fehlzeiten_tbl(r.id, uuid);

         UPDATE rg_fehlzeiten_tbl set
              fehlzeiten_monat = r.fehlm_4,
              fehlzeiten_jahr = r.fehlj_4,
              fehlzeiten_anzahl_fehltage = r.fehltg_4,
              fehlzeiten_unentschuldigt_tage = r.fehlutg_4,
              fehlzeiten_anzahl_fehlstunden = r.fehlst_4,
              fehlzeiten_unentschuldigt_stunden = r.fehlfust_4
         WHERE id = rgid;    
      END IF;


      IF r.fehlm_5 != '-1'
         OR r.fehlj_5 != '-1'
         OR r.fehltg_5 IS NOT NULL
         OR r.fehlutg_5 IS NOT NULL
         OR r.fehlst_5 IS NOT NULL
         OR r.fehlfust_5 IS NOT NULL THEN

         uuid = generate_uuidgen();
         rgid = create_rg_fehlzeiten_tbl(r.id, uuid);

         UPDATE rg_fehlzeiten_tbl set
              fehlzeiten_monat = r.fehlm_5,
              fehlzeiten_jahr = r.fehlj_5,
              fehlzeiten_anzahl_fehltage = r.fehltg_5,
              fehlzeiten_unentschuldigt_tage = r.fehlutg_5,
              fehlzeiten_anzahl_fehlstunden = r.fehlst_5,
              fehlzeiten_unentschuldigt_stunden = r.fehlfust_5
         WHERE id = rgid;    
      END IF;


      IF r.fehlm_6 != '-1'
         OR r.fehlj_6 != '-1'
         OR r.fehltg_6 IS NOT NULL
         OR r.fehlutg_6 IS NOT NULL
         OR r.fehlst_6 IS NOT NULL
         OR r.fehlfust_6 IS NOT NULL THEN

         uuid = generate_uuidgen();
         rgid = create_rg_fehlzeiten_tbl(r.id, uuid);

         UPDATE rg_fehlzeiten_tbl set
              fehlzeiten_monat = r.fehlm_6,
              fehlzeiten_jahr = r.fehlj_6,
              fehlzeiten_anzahl_fehltage = r.fehltg_6,
              fehlzeiten_unentschuldigt_tage = r.fehlutg_6,
              fehlzeiten_anzahl_fehlstunden = r.fehlst_6,
              fehlzeiten_unentschuldigt_stunden = r.fehlfust_6
         WHERE id = rgid;    
      END IF;


      IF r.fehlm_7 != '-1'
         OR r.fehlj_7 != '-1'
         OR r.fehltg_7 IS NOT NULL
         OR r.fehlutg_7 IS NOT NULL
         OR r.fehlst_7 IS NOT NULL
         OR r.fehlfust_7 IS NOT NULL THEN

         uuid = generate_uuidgen();
         rgid = create_rg_fehlzeiten_tbl(r.id, uuid);

         UPDATE rg_fehlzeiten_tbl set
              fehlzeiten_monat = r.fehlm_7,
              fehlzeiten_jahr = r.fehlj_7,
              fehlzeiten_anzahl_fehltage = r.fehltg_7,
              fehlzeiten_unentschuldigt_tage = r.fehlutg_7,
              fehlzeiten_anzahl_fehlstunden = r.fehlst_7,
              fehlzeiten_unentschuldigt_stunden = r.fehlfust_7
         WHERE id = rgid;    
      END IF;


      IF r.fehlm_8 != '-1'
         OR r.fehlj_8 != '-1'
         OR r.fehltg_8 IS NOT NULL
         OR r.fehlutg_8 IS NOT NULL
         OR r.fehlst_8 IS NOT NULL
         OR r.fehlfust_8 IS NOT NULL THEN

         uuid = generate_uuidgen();
         rgid = create_rg_fehlzeiten_tbl(r.id, uuid);

         UPDATE rg_fehlzeiten_tbl set
              fehlzeiten_monat = r.fehlm_8,
              fehlzeiten_jahr = r.fehlj_8,
              fehlzeiten_anzahl_fehltage = r.fehltg_8,
              fehlzeiten_unentschuldigt_tage = r.fehlutg_8,
              fehlzeiten_anzahl_fehlstunden = r.fehlst_8,
              fehlzeiten_unentschuldigt_stunden = r.fehlfust_8
         WHERE id = rgid;    
      END IF;


      IF r.fehlm_9 != '-1'
         OR r.fehlj_9 != '-1'
         OR r.fehltg_9 IS NOT NULL
         OR r.fehlutg_9 IS NOT NULL
         OR r.fehlst_9 IS NOT NULL
         OR r.fehlfust_9 IS NOT NULL THEN

         uuid = generate_uuidgen();
         rgid = create_rg_fehlzeiten_tbl(r.id, uuid);

         UPDATE rg_fehlzeiten_tbl set
              fehlzeiten_monat = r.fehlm_9,
              fehlzeiten_jahr = r.fehlj_9,
              fehlzeiten_anzahl_fehltage = r.fehltg_9,
              fehlzeiten_unentschuldigt_tage = r.fehlutg_9,
              fehlzeiten_anzahl_fehlstunden = r.fehlst_9,
              fehlzeiten_unentschuldigt_stunden = r.fehlfust_9
         WHERE id = rgid;    
      END IF;


      IF r.fehlm_10 != '-1'
         OR r.fehlj_10 != '-1'
         OR r.fehltg_10 IS NOT NULL
         OR r.fehlutg_10 IS NOT NULL
         OR r.fehlst_10 IS NOT NULL
         OR r.fehlfust_10 IS NOT NULL THEN

         uuid = generate_uuidgen();
         rgid = create_rg_fehlzeiten_tbl(r.id, uuid);

         UPDATE rg_fehlzeiten_tbl set
              fehlzeiten_monat = r.fehlm_10,
              fehlzeiten_jahr = r.fehlj_10,
              fehlzeiten_anzahl_fehltage = r.fehltg_10,
              fehlzeiten_unentschuldigt_tage = r.fehlutg_10,
              fehlzeiten_anzahl_fehlstunden = r.fehlst_10,
              fehlzeiten_unentschuldigt_stunden = r.fehlfust_10
         WHERE id = rgid;    
      END IF;


      IF r.fehlm_11 != '-1'
         OR r.fehlj_11 != '-1'
         OR r.fehltg_11 IS NOT NULL
         OR r.fehlutg_11 IS NOT NULL
         OR r.fehlst_11 IS NOT NULL
         OR r.fehlfust_11 IS NOT NULL THEN

         uuid = generate_uuidgen();
         rgid = create_rg_fehlzeiten_tbl(r.id, uuid);

         UPDATE rg_fehlzeiten_tbl set
              fehlzeiten_monat = r.fehlm_11,
              fehlzeiten_jahr = r.fehlj_11,
              fehlzeiten_anzahl_fehltage = r.fehltg_11,
              fehlzeiten_unentschuldigt_tage = r.fehlutg_11,
              fehlzeiten_anzahl_fehlstunden = r.fehlst_11,
              fehlzeiten_unentschuldigt_stunden = r.fehlfust_11
         WHERE id = rgid;    
      END IF;


      IF r.fehlm_12 != '-1'
         OR r.fehlj_12 != '-1'
         OR r.fehltg_12 IS NOT NULL
         OR r.fehlutg_12 IS NOT NULL
         OR r.fehlst_12 IS NOT NULL
         OR r.fehlfust_12 IS NOT NULL THEN

         uuid = generate_uuidgen();
         rgid = create_rg_fehlzeiten_tbl(r.id, uuid);

         UPDATE rg_fehlzeiten_tbl set
              fehlzeiten_monat = r.fehlm_12,
              fehlzeiten_jahr = r.fehlj_12,
              fehlzeiten_anzahl_fehltage = r.fehltg_12,
              fehlzeiten_unentschuldigt_tage = r.fehlutg_12,
              fehlzeiten_anzahl_fehlstunden = r.fehlst_12,
              fehlzeiten_unentschuldigt_stunden = r.fehlfust_12
         WHERE id = rgid;    
      END IF;


      IF r.fehlm_13 != '-1'
         OR r.fehlj_13 != '-1'
         OR r.fehltg_13 IS NOT NULL
         OR r.fehlutg_13 IS NOT NULL
         OR r.fehlst_13 IS NOT NULL
         OR r.fehlfust_13 IS NOT NULL THEN

         uuid = generate_uuidgen();
         rgid = create_rg_fehlzeiten_tbl(r.id, uuid);

         UPDATE rg_fehlzeiten_tbl set
              fehlzeiten_monat = r.fehlm_13,
              fehlzeiten_jahr = r.fehlj_13,
              fehlzeiten_anzahl_fehltage = r.fehltg_13,
              fehlzeiten_unentschuldigt_tage = r.fehlutg_13,
              fehlzeiten_anzahl_fehlstunden = r.fehlst_13,
              fehlzeiten_unentschuldigt_stunden = r.fehlfust_13
         WHERE id = rgid;    
      END IF;


      IF r.fehlm_14 != '-1'
         OR r.fehlj_14 != '-1'
         OR r.fehltg_14 IS NOT NULL
         OR r.fehlutg_14 IS NOT NULL
         OR r.fehlst_14 IS NOT NULL
         OR r.fehlfust_14 IS NOT NULL THEN

         uuid = generate_uuidgen();
         rgid = create_rg_fehlzeiten_tbl(r.id, uuid);

         UPDATE rg_fehlzeiten_tbl set
              fehlzeiten_monat = r.fehlm_14,
              fehlzeiten_jahr = r.fehlj_14,
              fehlzeiten_anzahl_fehltage = r.fehltg_14,
              fehlzeiten_unentschuldigt_tage = r.fehlutg_14,
              fehlzeiten_anzahl_fehlstunden = r.fehlst_14,
              fehlzeiten_unentschuldigt_stunden = r.fehlfust_14
         WHERE id = rgid;    
      END IF;
      

      IF r.fehlm_15 != '-1'
         OR r.fehlj_15 != '-1'
         OR r.fehltg_15 IS NOT NULL
         OR r.fehlutg_15 IS NOT NULL
         OR r.fehlst_15 IS NOT NULL
         OR r.fehlfust_15 IS NOT NULL THEN


         uuid = generate_uuidgen();
         rgid = create_rg_fehlzeiten_tbl(r.id, uuid);

         UPDATE rg_fehlzeiten_tbl set
              fehlzeiten_monat = r.fehlm_15,
              fehlzeiten_jahr = r.fehlj_15,
              fehlzeiten_anzahl_fehltage = r.fehltg_15,
              fehlzeiten_unentschuldigt_tage = r.fehlutg_15,
              fehlzeiten_anzahl_fehlstunden = r.fehlst_15,
              fehlzeiten_unentschuldigt_stunden = r.fehlfust_15
         WHERE id = rgid;    
      END IF;


      IF r.fehlm_16 != '-1'
         OR r.fehlj_16 != '-1'
         OR r.fehltg_16 IS NOT NULL
         OR r.fehlutg_16 IS NOT NULL
         OR r.fehlst_16 IS NOT NULL
         OR r.fehlfust_16 IS NOT NULL THEN

         uuid = generate_uuidgen();
         rgid = create_rg_fehlzeiten_tbl(r.id, uuid);

         UPDATE rg_fehlzeiten_tbl set
              fehlzeiten_monat = r.fehlm_16,
              fehlzeiten_jahr = r.fehlj_16,
              fehlzeiten_anzahl_fehltage = r.fehltg_16,
              fehlzeiten_unentschuldigt_tage = r.fehlutg_16,
              fehlzeiten_anzahl_fehlstunden = r.fehlst_16,
              fehlzeiten_unentschuldigt_stunden = r.fehlfust_16
         WHERE id = rgid;    
      END IF;


      IF r.fehlm_17 != '-1'
         OR r.fehlj_17 != '-1'
         OR r.fehltg_17 IS NOT NULL
         OR r.fehlutg_17 IS NOT NULL
         OR r.fehlst_17 IS NOT NULL
         OR r.fehlfust_17 IS NOT NULL THEN

         uuid = generate_uuidgen();
         rgid = create_rg_fehlzeiten_tbl(r.id, uuid);

         UPDATE rg_fehlzeiten_tbl set
              fehlzeiten_monat = r.fehlm_17,
              fehlzeiten_jahr = r.fehlj_17,
              fehlzeiten_anzahl_fehltage = r.fehltg_17,
              fehlzeiten_unentschuldigt_tage = r.fehlutg_17,
              fehlzeiten_anzahl_fehlstunden = r.fehlst_17,
              fehlzeiten_unentschuldigt_stunden = r.fehlfust_17
         WHERE id = rgid;    
      END IF;


      IF r.fehlm_18 != '-1'
         OR r.fehlj_18 != '-1'
         OR r.fehltg_18 IS NOT NULL
         OR r.fehlutg_18 IS NOT NULL
         OR r.fehlst_18 IS NOT NULL
         OR r.fehlfust_18 IS NOT NULL THEN

         uuid = generate_uuidgen();
         rgid = create_rg_fehlzeiten_tbl(r.id, uuid);

         UPDATE rg_fehlzeiten_tbl set
              fehlzeiten_monat = r.fehlm_18,
              fehlzeiten_jahr = r.fehlj_18,
              fehlzeiten_anzahl_fehltage = r.fehltg_18,
              fehlzeiten_unentschuldigt_tage = r.fehlutg_18,
              fehlzeiten_anzahl_fehlstunden = r.fehlst_18,
              fehlzeiten_unentschuldigt_stunden = r.fehlfust_18
         WHERE id = rgid;    
      END IF;

      -- Falls keine RGs angelegt wurden, eine neue erstellen
      SELECT COUNT(id) INTO rg_count FROM rg_fehlzeiten_tbl WHERE master_id  = r.id;
      IF rg_count = 0 THEN
      	uuid = generate_uuidgen();
        rgid = create_rg_fehlzeiten_tbl(r.id, uuid);
      END IF;

  END LOOP;
END;
$$ LANGUAGE plpgsql;
SET ROLE postgres;
SELECT insert_rg();
SET ROLE :db_adm;

-- Restore old function with its corresponding exception
CREATE OR REPLACE FUNCTION create_rg_fehlzeiten_tbl(pid integer, nuuid char(36))
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid integer;
    mid integer;
BEGIN
    mid := pid;

    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_fehlzeiten_tbl (master_id) VALUES (pid);

    SELECT INTO rid currval('rg_fehlzeiten_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_fehlzeiten_tbl
        SET    uuid_id = nuuid
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_fehlzeiten_tbl', rid);
        
    END IF;

    RETURN rid;
END;
$$;

SET ROLE postgres;
ALTER TABLE master_tbl ENABLE TRIGGER USER;

-- ROLLBACK;
COMMIT;
