BEGIN;

-- 
-- Repeat Group fuer probleme 5.2.2 Probleme
-- 

ALTER TABLE master_tbl DISABLE TRIGGER USER;
SET ROLE :db_adm;

INSERT INTO db_history (beschreibung, zeitpunkt)
       VALUES ('2.0/03_rg_probleme.sql', current_timestamp);


-- Create a new function to generate RG-entries.
-- This is the original function, without the DS IS check and exception
CREATE OR REPLACE FUNCTION create_rg_probleme_tbl(pid integer, nuuid char(36))
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid integer;
    mid integer;
BEGIN
    mid := pid;

    IF NOT isEditorOrStandin(mid) THEN
        --RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_probleme_tbl (master_id) VALUES (pid);

    SELECT INTO rid currval('rg_probleme_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_probleme_tbl
        SET    uuid_id = nuuid
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_probleme_tbl', rid);
        
    END IF;

    RETURN rid;
END;
$$;


-- Uebernahme der alten Eintraege in eine Repeat Group
--
-- Fuer alle Eintrage in der master_tbl
CREATE OR REPLACE FUNCTION insert_rg() RETURNS VOID AS $$
DECLARE
  r RECORD;
  rgid INTEGER;
  rg_count INTEGER;
  uuid TEXT;
BEGIN
  FOR r in SELECT *
  FROM master_tbl LOOP

      -- Zunaechst alle RGs loeschen
      DELETE from rg_probleme_tbl WHERE master_id = r.id;

      -- probleme_a -> "... des jungen Menschen", ID: 1 
      IF r.probleme_a IS NOT NULL THEN
	    uuid = generate_uuidgen();
            rgid = create_rg_probleme_tbl(r.id, uuid);
            UPDATE rg_probleme_tbl set
                  probleme_aus_sicht_von = 1,
                  probleme_beschreibung =  r.probleme_a
            WHERE id = rgid;
      END IF;

      
      -- probleme_b -> "...der Eltern/Sorgeberechtigten", ID: 2 
      IF r.probleme_b IS NOT NULL THEN
	    uuid = generate_uuidgen();
            rgid = create_rg_probleme_tbl(r.id, uuid);
            UPDATE rg_probleme_tbl set
                  probleme_aus_sicht_von = 2,
                  probleme_beschreibung =  r.probleme_b
            WHERE id = rgid;
      END IF;


      -- probleme_c -> "...der Koordinierungsstelle", ID: 3 
      IF r.probleme_c IS NOT NULL THEN
	    uuid = generate_uuidgen();
            rgid = create_rg_probleme_tbl(r.id, uuid);
            UPDATE rg_probleme_tbl set
                  probleme_aus_sicht_von = 3,
                  probleme_beschreibung =  r.probleme_c
            WHERE id = rgid;
      END IF;

      -- probleme_d -> "...der Lehrerin/des Lehrers", ID: 4 
      IF r.probleme_d IS NOT NULL THEN
	    uuid = generate_uuidgen();
            rgid = create_rg_probleme_tbl(r.id, uuid);
            UPDATE rg_probleme_tbl set
                  probleme_aus_sicht_von = 4,
                  probleme_beschreibung =  r.probleme_d
            WHERE id = rgid;
      END IF;
      -- probleme_e -> "...des Jugendamtes", ID: 5
      IF r.probleme_e IS NOT NULL THEN
	    uuid = generate_uuidgen();
            rgid = create_rg_probleme_tbl(r.id, uuid);
            UPDATE rg_probleme_tbl set
                  probleme_aus_sicht_von = 5,
                  probleme_beschreibung =  r.probleme_e
            WHERE id = rgid;
      END IF;

      -- probleme_f -> "Sonstiges", ID: -1
      IF r.probleme_f_t  IS NOT NULL
      	OR r.probleme_f IS NOT NULL THEN
	    uuid = generate_uuidgen();
            rgid = create_rg_probleme_tbl(r.id, uuid);
            UPDATE rg_probleme_tbl set
                  probleme_aus_sicht_von = -2,
                  probleme_beschreibung =  array_to_string(ARRAY[r.probleme_f, r.probleme_f_t], ';')
	    WHERE id = rgid;
      END IF;


      -- Falls keine RGs angelegt wurden, eine neue erstellen
      SELECT COUNT(id) INTO rg_count FROM rg_probleme_tbl WHERE master_id  = r.id;
      IF rg_count = 0 THEN
      	uuid = generate_uuidgen();
        rgid = create_rg_probleme_tbl(r.id, uuid);
      END IF;

  END LOOP;
END;
$$ LANGUAGE plpgsql;
SET ROLE postgres;
SELECT insert_rg();
SET ROLE :db_adm;
DROP FUNCTION insert_rg();




-- Restore old function with its corresponding exception
CREATE OR REPLACE FUNCTION create_rg_probleme_tbl(pid integer, nuuid char(36))
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid integer;
    mid integer;
BEGIN
    mid := pid;

    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_probleme_tbl (master_id) VALUES (pid);

    SELECT INTO rid currval('rg_probleme_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_probleme_tbl
        SET    uuid_id = nuuid
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_probleme_tbl', rid);
        
    END IF;

    RETURN rid;
END;
$$;



SET ROLE postgres;
ALTER TABLE master_tbl ENABLE TRIGGER USER;

-- ROLLBACK;
COMMIT;
