BEGIN;

-- 
-- Repeat Group fuer 3.6 Bezugspersonen
-- 

ALTER TABLE master_tbl DISABLE TRIGGER USER;

SET ROLE :db_adm;

INSERT INTO db_history (beschreibung, zeitpunkt)
       VALUES ('2.0/04_rg_bezugspersonen.sql', current_timestamp);


-- Create a new function to generate RG-entries.
-- This is the original function, without the DS IS check and exception
CREATE OR REPLACE FUNCTION create_rg_kontaktdaten_tbl(pid integer, nuuid char(36))
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid integer;
    mid integer;
BEGIN
    mid := pid;

    IF NOT isEditorOrStandin(mid) THEN
        -- RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_kontaktdaten_tbl (master_id) VALUES (pid);

    SELECT INTO rid currval('rg_kontaktdaten_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_kontaktdaten_tbl
        SET    uuid_id = nuuid
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_kontaktdaten_tbl', rid);
        
    END IF;

    RETURN rid;
END;
$$;


-- Uebernahme der alten Eintraege in eine Repeat Group
--
-- Fuer alle Eintrage in der master_tbl
CREATE OR REPLACE FUNCTION insert_rg() RETURNS VOID AS $$
DECLARE
  r RECORD;
  rgid INTEGER;
  rg_count INTEGER;
  uuid TEXT;
BEGIN

  FOR r in SELECT *
  FROM master_tbl LOOP
      -- Zunaechst alle RGs loeschen
      DELETE from rg_kontaktdaten_tbl WHERE master_id = r.id;

      -- Kontaktdaten Bezugsperson 1
      IF r.kd_bezug_name_1 IS NOT NULL
            OR r.kd_bezug_funktion_1 != -1
            OR r.kd_bezug_vorname_1 IS NOT NULL
            OR r.kd_bezug_strasse_1 IS NOT NULL
            OR r.kd_bezug_strassenr_1 IS NOT NULL
            OR r.kd_bezug_ort_1 IS NOT NULL
            OR r.kd_bezug_plz_1 IS NOT NULL
            OR r.kd_bezug_1_vorwahl IS NOT NULL
            OR r.kd_bezug_1_telefon IS NOT NULL
            OR r.kd_bezug_email_1 IS NOT NULL THEN

            uuid = generate_uuidgen();
            rgid = create_rg_kontaktdaten_tbl(r.id, uuid);
            UPDATE rg_kontaktdaten_tbl set
               kontaktdaten_bezugsperson_name = r.kd_bezug_name_1,
               kontaktdaten_bezugsperson_vorname = r.kd_bezug_vorname_1,
               kontaktdaten_bezugsperson_funktion = r.kd_bezug_funktion_1,
               kontaktdaten_bezugsperson_strasse = r.kd_bezug_strasse_1,
               kontaktdaten_bezugsperson_strassenr = r.kd_bezug_strassenr_1,
               kontaktdaten_bezugsperson_ort = r.kd_bezug_ort_1,
               kontaktdaten_bezugsperson_plz = r.kd_bezug_plz_1,
               kontaktdaten_bezugsperson_vorwahl = r.kd_bezug_1_vorwahl,
               kontaktdaten_bezugsperson_telefon = r.kd_bezug_1_telefon,
               kontaktdaten_bezugsperson_email = r.kd_bezug_email_1
            WHERE id = rgid;
      END IF;

      -- Kontaktdaten Bezugsperson 2
      IF r.kd_bezug_name_2 IS NOT NULL
            OR r.kd_bezug_funktion_2 != -1
            OR r.kd_bezug_vorname_2 IS NOT NULL
            OR r.kd_bezug_strasse_2 IS NOT NULL
            OR r.kd_bezug_strassenr_2 IS NOT NULL
            OR r.kd_bezug_ort_2 IS NOT NULL
            OR r.kd_bezug_plz_2 IS NOT NULL
            OR r.kd_bezug_2_vorwahl IS NOT NULL
            OR r.kd_bezug_2_telefon IS NOT NULL
            OR r.kd_bezug_email_2 IS NOT NULL THEN

	    uuid = generate_uuidgen();
            rgid = create_rg_kontaktdaten_tbl(r.id, uuid);
            UPDATE rg_kontaktdaten_tbl set
               kontaktdaten_bezugsperson_name = r.kd_bezug_name_2,
               kontaktdaten_bezugsperson_vorname = r.kd_bezug_vorname_2,
               kontaktdaten_bezugsperson_funktion = r.kd_bezug_funktion_2,
               kontaktdaten_bezugsperson_strasse = r.kd_bezug_strasse_2,
               kontaktdaten_bezugsperson_strassenr = r.kd_bezug_strassenr_2,
               kontaktdaten_bezugsperson_ort = r.kd_bezug_ort_2,
               kontaktdaten_bezugsperson_plz = r.kd_bezug_plz_2,
               kontaktdaten_bezugsperson_vorwahl = r.kd_bezug_2_vorwahl,
               kontaktdaten_bezugsperson_telefon = r.kd_bezug_2_telefon,
               kontaktdaten_bezugsperson_email = r.kd_bezug_email_2
            WHERE id = rgid;
      END IF;

       -- Kontaktdaten Bezugsperson 3
      IF r.kd_bezug_name_3 IS NOT NULL
            OR r.kd_bezug_funktion_3 != -1
            OR r.kd_bezug_vorname_3 IS NOT NULL
            OR r.kd_bezug_strasse_3 IS NOT NULL
            OR r.kd_bezug_strassenr_3 IS NOT NULL
            OR r.kd_bezug_ort_3 IS NOT NULL
            OR r.kd_bezug_plz_3 IS NOT NULL
            OR r.kd_bezug_3_vorwahl IS NOT NULL
            OR r.kd_bezug_3_telefon IS NOT NULL
            OR r.kd_bezug_email_3 IS NOT NULL THEN

	    uuid = generate_uuidgen();
            rgid = create_rg_kontaktdaten_tbl(r.id, uuid);
            UPDATE rg_kontaktdaten_tbl set
               kontaktdaten_bezugsperson_name = r.kd_bezug_name_3,
               kontaktdaten_bezugsperson_vorname = r.kd_bezug_vorname_3,
               kontaktdaten_bezugsperson_funktion = r.kd_bezug_funktion_3,
               kontaktdaten_bezugsperson_strasse = r.kd_bezug_strasse_3,
               kontaktdaten_bezugsperson_strassenr = r.kd_bezug_strassenr_3,
               kontaktdaten_bezugsperson_ort = r.kd_bezug_ort_3,
               kontaktdaten_bezugsperson_plz = r.kd_bezug_plz_3,
               kontaktdaten_bezugsperson_vorwahl = r.kd_bezug_3_vorwahl,
               kontaktdaten_bezugsperson_telefon = r.kd_bezug_3_telefon,
               kontaktdaten_bezugsperson_email = r.kd_bezug_email_3
            WHERE id = rgid;
      END IF;

      -- Falls keine RGs angelegt wurden, eine neue erstellen
      SELECT COUNT(id) INTO rg_count FROM rg_kontaktdaten_tbl WHERE master_id  = r.id;
      IF rg_count = 0 THEN
      	uuid = generate_uuidgen();
        rgid = create_rg_kontaktdaten_tbl(r.id, uuid);
      END IF;

  END LOOP;
END;
$$ LANGUAGE plpgsql;
SET ROLE postgres;
SELECT insert_rg();
SET ROLE :db_adm;
DROP FUNCTION insert_rg();




-- Restore old function with its corresponding exception
CREATE OR REPLACE FUNCTION create_rg_kontaktdaten_tbl(pid integer, nuuid char(36))
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid integer;
    mid integer;
BEGIN
    mid := pid;

    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_kontaktdaten_tbl (master_id) VALUES (pid);

    SELECT INTO rid currval('rg_kontaktdaten_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_kontaktdaten_tbl
        SET    uuid_id = nuuid
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_kontaktdaten_tbl', rid);
        
    END IF;

    RETURN rid;
END;
$$;


SET ROLE postgres;
ALTER TABLE master_tbl ENABLE TRIGGER USER;

-- ROLLBACK;
COMMIT;
