# -*- coding: utf-8 -*-

import sys
import logging
from datetime import datetime, timedelta, date
import traceback

import formencode

from formed.instance.backends.postgres import DBFactory as InstanceFactory
from formed.instance.tree import SetDataException

from mpulsweb.lib.translation import _
from mpulsweb.model.logbook import Logbook, LogbookEntry
from mpulsweb.lib.base import BaseController, c, g, render, request, session
from mpulsweb.lib.db import PostgresDBInterface
import mpulsweb.lib.helpers as h
from mpulsweb.lib.helper.case import copy_instance_tree, walk_it

from waskoweb.lib.validators import \
     ExtraTimeFormValidator1, ExtraTimeFormValidator2, \
     ReopenFormValidator1, ReopenFormValidator1abgel, \
     ReopenFormValidator2, ReopenFormValidator2abgel, \
     RenewFormValidator


log = logging.getLogger(__name__)


# items not to be copied from old case for "neuaufnahme" and
# "wiederaufnahme"
# erstgespraech, datum_cm_start are relevant for both.
# datum_kf_1, datum_bfplan only have to be treated specially for
# wiederaufnahme, but they won't be copied at all for neuaufnahme, so
# including them here should not affect neuaufnahme.
COPY_BLACK_LIST = frozenset(["erstgespraech", "datum_cm_start",
                             "datum_kf_1", "datum_bfplan"])


def toErrorDicts(e):
    values, errors = {}, {}
    for key, err in e.errors.iteritems():
        key = key.split(':')[0]
        for e in err:
            values[key] = e.bad
            errors[key] = e.msg
    return values, errors

def create_logbook(id=None, header=None, text=None):
    if id is None:
        id = session['case'].id
    logbook = Logbook(id)
    entry = LogbookEntry()
    entry.datum = datetime.now()
    entry.art = 0
    entry.kurz_notiz = header
    entry.notiz = text
    logbook.addEntry(entry)
    logbook.store(session['USER_AUTHORIZED'].id)

def mark_case(id, mark=None):
    data = {}
    if mark:
        factory = InstanceFactory(g.formedTree, PostgresDBInterface())
        it = factory.loadInstanceTreeByIdentifier(id)
        item = it.getItem('fn:%s' % id)
        fn = item.getValue()
        if fn:
            fn += '_%s' % mark
            data['fn:%s' % id] = fn
        else:
            data['fn:%s' % id] = '_%s' % mark
        it.setData(data)
        it.commit()


def doc_abschluss(id, to_id=None):
    if to_id is None:
        to_id = id
    data = {}
    factory = InstanceFactory(g.formedTree, PostgresDBInterface())
    oit = factory.loadInstanceTreeByIdentifier(id)
    nit = factory.loadInstanceTreeByIdentifier(to_id)

    data['cm_dok_end_art:%s' % to_id] = str(_get_value(oit, 'cm_end_art', id))
    data['cm_dok_end_verm:%s' % to_id] = str(_get_value(oit, 'cm_end_verm', id))
    data['cm_dok_end_verm_berufsschule:%s' % to_id] = str(_get_value(oit, 'cm_end_verm_berufsschule', id))
    data['cm_dok_end_sonst:%s' % to_id] = str(_get_value(oit,'cm_end_sonst',id))
    data['cm_dok_datum_beendigung:%s' % to_id] = \
        h.dd_mm_YYYY(_get_value(oit, 'datum_cm_ende', id))

    nit.setData(data)
    nit.commit()
    log.debug(data)

def save_formdata(id, data):
    # Füge alte params aus dem Dialog in der verwaltung hinzu und
    # schreibe alles weg
    id = int(id)
    factory = InstanceFactory(g.formedTree, PostgresDBInterface())
    it = factory.loadInstanceTreeByIdentifier(id)
    sparams = session.get('lifetimeparams', {})
    cm2c_t = _get_value(it, 'cm2c_t', id)
    data.update(sparams)
    _set_data(it, data, id)

def _convert4Formed(data, id):
    convertet_data = {}
    for key, value in data.iteritems():
        if key == 'cm2c_t2':
            continue
        if isinstance(value, date):
            value = h.format_date(value)
        convertet_data[":".join([str(key), str(id)])] = value
    return convertet_data

def _set_data(it, data, case_id, do_write=True):
    convertet_data = _convert4Formed(data, case_id)
    it.setData(convertet_data)
    if do_write:
        it.commit()

def _get_value(it, id, case_id):
    real_id = "%s:%s" % (id, case_id)
    item = it.getItem(real_id)
    if item:
        value = item.getValue()
        log.debug('%s = "%r"' % (real_id, value))
        return value
    log.error('Can not find item "%s"' % real_id)

def checkcase(it, id):
    # Check if case is anonymizable
    case = g.case_factory.loadById(id)
    try:
        case.check_anonymizeability()
        c.is_anonymizable = True
    except:
        c.is_anonymizable = False

    case_id = id
    D = _get_value

    end_art = D(it, 'cm_end_art', case_id)
    c.phase = D(it, 'phase', case_id)
    c.cm_neuauf_genehm = D(it, 'cm_neuauf_genehm', case_id)

    ## Welche Beendigung?
    c.erfolgreiche_reintegration = False
    c.sonstige_beendigung = False
    if end_art == 1 or end_art == 4:
        c.erfolgreiche_reintegration = True
    elif end_art == 2:
        c.sonstige_beendigung = True

    # Wieviele Wiederholungen?
    log.debug('Wiederaufnahme1: %s' % D(it, 'cm_wiederauf_genehm_1', case_id))
    log.debug('Wiederaufnahme2: %s' % D(it, 'cm_wiederauf_genehm_2', case_id))
    if D(it, 'cm_wiederauf_genehm_2', case_id) in (1, 0):
        c.num_wiederaufnahme = 2
    elif D(it, 'cm_wiederauf_genehm_1', case_id) in (1,):
        c.num_wiederaufnahme = 1
    else:
        c.num_wiederaufnahme = 0

    # Wieviele Verlängerungen?
    if D(it, 'cm_datum_verlaengerung_2', case_id):
        c.num_verlaengerung = 2
    elif D(it, 'cm_datum_verlaengerung_1', case_id):
        c.num_verlaengerung = 1
    else:
        c.num_verlaengerung = 0
    log.debug('Anzahl Verlaengerungen: %s' % c.num_verlaengerung)


    # Wurde Wiederaufnahme abgelehnt? -> alle Optionen ausblenden
    if (D(it, 'cm_wiederauf_genehm_1', case_id) in (0,)
        or D(it, 'cm_wiederauf_genehm_2', case_id) in (0,)):
        c.disable_all = True
        c.wiederaufn_abg = True
    else:
        c.disable_all = False
        c.wiederaufn_abg = False


    # Verlängerung anzeigen
    if (not c.disable_all and c.phase in (2,) and c.num_verlaengerung < 2
        and c.num_wiederaufnahme < 1):
        c.show_verlaengerung = True
    else:
        c.show_verlaengerung = False

    # Wiedervorlage anzeigen
    if (not c.disable_all
        and (c.phase in (3, 4, 5)
              and c.erfolgreiche_reintegration
              and c.num_wiederaufnahme < 1
              and c.is_anonymizable)
        or (c.num_wiederaufnahme == 1 and c.phase not in (-1, 0, 1, 3, 4, 5))):
        c.show_wiedervorlage = True
    else:
        c.show_wiedervorlage = False

    # Neuaufnahme anzeigen
    if (not c.disable_all and (c.phase in (3, 4, 5)) and c.sonstige_beendigung
        and (c.num_wiederaufnahme < 1) and (c.cm_neuauf_genehm not in (0, 1))
        and c.is_anonymizable):
        c.show_neuaufnahme = True
    else:
        c.show_neuaufnahme = False

    # Abbruch anzeigen
    if not c.disable_all and c.phase in (-1, 0, 1):
        c.show_abort = True
    else:
        c.show_abort = False


class CaselifetimeController(BaseController):

    def index(self, id=None):
        c.form_errors = {}
        c.form_result = {}
        c.form_defaults = {}
        factory = InstanceFactory(g.formedTree, PostgresDBInterface())
        it = factory.loadInstanceTreeByIdentifier(id)
        case_id = it.getRootNode().getIdentifier()
        checkcase(it, case_id)
        D = _get_value

        # Verlängerungen
        ende_verl1 = D(it, 'cm_datum_verlaengerung_1', case_id)
        ende_verl2 = D(it, 'cm_datum_verlaengerung_2', case_id)
        if c.num_verlaengerung < 1:
            begin_cm = D(it, 'datum_cm_start', case_id)
            if begin_cm and not ende_verl1:
                ende_verl1 = begin_cm+timedelta(days=458)
        if c.num_verlaengerung == 1 and not ende_verl2:
                ende_verl2 = ende_verl1+timedelta(days=31*3)

        c.form_defaults['datum_cm_ende'] = h.dd_mm_YYYY(D(it, 'datum_cm_ende',
                                                          case_id))
        c.form_defaults['cm_datum_verlaengerung_1'] = h.dd_mm_YYYY(ende_verl1)
        c.form_defaults['cm_bestaet_verlaengerung_1'] = \
            D(it, 'cm_bestaet_verlaengerung_1', case_id)
        c.form_defaults['cm_anmerk_verlaengerung_1'] = \
            D(it, 'cm_anmerk_verlaengerung_1', case_id)
        c.form_defaults['cm_datum_verlaengerung_2'] = h.dd_mm_YYYY(ende_verl2)
        c.form_defaults['cm_bestaet_verlaengerung_2'] = \
            D(it, 'cm_bestaet_verlaengerung_2', case_id)
        c.form_defaults['cm_anmerk_verlaengerung_2'] = \
            D(it, 'cm_anmerk_verlaengerung_2', case_id)

        # Wiederaufnahme
        wieder1_date = D(it, 'cm_wiederauf_datum_bis_1', case_id)
        wieder2_date = D(it, 'cm_wiederauf_datum_bis_2', case_id)
        if c.num_wiederaufnahme >= 1:
            if wieder1_date and not wieder2_date:
                wieder2_date = wieder1_date+timedelta(days=31*3)

        c.form_defaults['cm_wiederauf_genehm_1'] = \
            D(it, 'cm_wiederauf_genehm_1', case_id)
        c.form_defaults['cm_wiederauf_genehm_datum_1'] = \
            h.dd_mm_YYYY(D(it, 'cm_wiederauf_genehm_datum_1', case_id))
        c.form_defaults['cm_wiederauf_datum_bis_1'] = h.dd_mm_YYYY(wieder1_date)
        c.form_defaults['cm_wiederauf_genehm_2'] = \
            D(it, 'cm_wiederauf_genehm_2', case_id)
        c.form_defaults['cm_wiederauf_datum_bis_2'] = \
            h.dd_mm_YYYY(wieder2_date)
        c.form_defaults['cm2c_t'] = D(it, 'cm2c_t', case_id)

        form = render('/caselifetime/overview.mako')
        return formencode.htmlfill.render(form,
                                          defaults=c.form_defaults)

    def changeLifetime(self):
        if request.params.has_key('wiederaufnahme'):
            return self.reopenAction()
        elif request.params.has_key('verlaengerung'):
            return self.extraTimeAction()
        elif request.params.has_key('neuaufnahme'):
            return self.renewAction()
        elif request.params.has_key('abort'):
            return self.abortAction(session['case'].id)
        else:
            pass
        return render('/caselifetime/overview.mako')

    def abortcase(self, id, confirmed='0'):
        if confirmed == '1':
            # mark case for deletion
            case = g.case_factory.loadById(id)
            case.setState(3)
            c.success_for = _(u'Entry documentation aborted.')
            c.success_text =  _(u'''\
The entry documentation was successfully aborted. The case document was given
to the administration for deletion. Click on OK to get back to the
case documents overview.''')
            c.url_ok = h.url(controller="case_overview", action='index')
            return render('caselifetime/dialogs/success_reopen.mako')
        else:
            c.confirm_for = _(u'Do you want to abort the entry documentation?')
            c.question = _(u'''\
Do you really want to abort the entry documentation and free the case document
for deletion? The case document will not be editable then and will be given to
the administration for seletion.''')

            c.url_yes = h.url(controller="caselifetime", action="abortcase", 
                              id=id, confirmed="1")
            c.url_no = h.url(controller="caselifetime", action="index",
                             id=id)
            return render('caselifetime/dialogs/confirm_reopen.mako')

    def abortAction(self, id):
        return self.abortcase(id)
        return self.index(id)

    def extraTimeAction(self):
        factory = InstanceFactory(g.formedTree, PostgresDBInterface())
        it = factory.loadInstanceTreeByIdentifier(session['case'].id)
        case_id = it.getRootNode().getIdentifier()
        S = _set_data
        checkcase(it, case_id)

        form_result = {}
        form_errors = {}

        if c.num_verlaengerung < 1:
            validator = ExtraTimeFormValidator1()
        else:
            validator = ExtraTimeFormValidator2()

        try:
            # 1. Check. Formencode check general format of date and if value is
            # present
            form_result = validator.to_python(request.params)
            try:
                # Save method will also do semantic check (dates or in
                # chronology correct order)
                S(it, form_result, case_id)
                create_logbook(header=_('Extension of the integration process'),
                               text=_('For this case a extension of the '
                                       'integration process was documented.'))
                return self.index(session['case'].id)
            except SetDataException, err:
                form_result_1, form_errors = toErrorDicts(err)
        except formencode.Invalid, error:
            form_result = error.value
            form_errors = error.error_dict or {}

        form_errors = form_errors
        form_result = form_result
        form = render('/caselifetime/overview.mako')
        return formencode.htmlfill.render(form,
                                          defaults=form_result,
                                          errors=form_errors)

    def renew_ok(self, id, confirmed='0'):
        factory = InstanceFactory(g.formedTree, PostgresDBInterface())
        if confirmed == '1':
            data = {}
            new_case = None
            try:
                it = factory.loadInstanceTreeByIdentifier(id)
                # Lege neuen Fall an, es sollen aber nur bestimmte
                # Felder kopiert werden
                fields = []
                for section in ['ee', 'phase-a', 'group-5']:
                    node = it.getItem("%s:%s" % (section, id))
                    for field in walk_it(node):
                        name = field.getIdentifier().split(':')[0]
                        if name not in COPY_BLACK_LIST:
                            fields.append(name)
                        else:
                            log.info('Not copying %s as it is in'
                                     ' COPY_BLACK_LIST' % name)
                new_it = copy_instance_tree(it, fields)
                new_id = new_it.getRootNode().getIdentifier()
                data['phase'] = '-1'
                data['cm2c'] = '1'
                doc_abschluss(id, new_id)
                # Dokumentiere Neuaufnahme in neuem Fall
                save_formdata(new_id, data)
                mark_case(new_id, 'NA')
                create_logbook(id=new_id, header=_('Addition'),
                               text=_('For this case an addition of the '
                                       'integration process was documented.'))

                # markiere den alten Fall zur Anonymisierung
                old_case = g.case_factory.loadById(id)
                old_case.setState(4)
                mark_case(old_case.id, 'NAU')
                create_logbook(header=_('Addition'),
                               text=_('For this case an addition of the '
                                       'integration process was documented.'))
            except Exception, err:
                traceback.print_exc(file=sys.stderr)

            c.success_for = _(u"The case document was renewed.")
            c.success_text =  _(u'''\
The case document was renewed. Click on OK to open the new case document.
Please check the data and update them, if needed. The old case document
was given to the administration for Anonymizing.''')
            c.url_ok = h.url(controller="case",action="select", id=new_id)
            return render('caselifetime/dialogs/success_reopen.mako')
        else:
            c.confirm_for = _(u"Do you want to renew the case?")
            c.question = _(u"""\
Do you want to renew the case? Then the current case document will be
anonymized and important parts of phase A and phase B will be copied into a
new case document. You will work on the new case document then.""")
            c.url_yes = h.url(controller="caselifetime", action="renew_ok",
                              id=id, confirmed="1")
            c.url_no = h.url(controller="caselifetime", action="index", id=id)
            return render('caselifetime/dialogs/confirm_reopen.mako')

    def renew_notok(self, id, confirmed='0'):
        if confirmed == '1':
            data = {}
            try:
                save_formdata(id, data)
                create_logbook(id=id, header=_('Addition'),
                               text=_('For this case a not accepted addition '
                                       'of the integration process '
                                       'was documented.'))

            except Exception, err:
                log.exception('Error in renew_notok')

            c.success_for = _(u"Not accepted new subscription documented.")
            c.success_text = _(u'''\
The not accepted new subscription was documented. Click on OK to get to the
entry documentation of the case document.''')
            c.url_ok = h.url(controller="formularpage",
                                 action="select",
                                 id="page-0:%s" % id)
            return render('caselifetime/dialogs/success_reopen.mako')
        else:
            c.confirm_for = _(u"Do you want to document the not accepted new subscription?")
            c.question =  _(u"""\
You entered, that no acceptence of the new subscription is available.
If you continue, no new addition of this case will be possible.""")
            c.url_yes = h.url(controller="caselifetime", action="renew_notok",
                              id=id, confirmed="1")
            c.url_no = h.url(controller="caselifetime",action="index", id=id)
            return render('caselifetime/dialogs/confirm_reopen.mako')

    def reopen_ok(self, id, confirmed='0'):
        factory = InstanceFactory(g.formedTree, PostgresDBInterface())
        it = factory.loadInstanceTreeByIdentifier(id)
        checkcase(it, id)
        if confirmed == '1':
            try:
                data = session['lifetimeparams']
                if c.num_wiederaufnahme < 1:
                    # Lege neuen Fall an, es sollen aber nur bestimmte
                    # Felder kopiert werden
                    fields = []
                    for section in ['ee', 'phase-a', 'group-5', 'group-18',
                                    'group-27']:
                        node = it.getItem("%s:%s" % (section, id))
                        for field in walk_it(node):
                            name = field.getIdentifier().split(':')[0]
                            if name not in COPY_BLACK_LIST:
                                fields.append(name)
                            else:
                                log.info('Not copying %s as it is in'
                                         ' COPY_BLACK_LIST' % name)
                    new_it = copy_instance_tree(it, fields)
                    new_id = new_it.getRootNode().getIdentifier()

                    doc_abschluss(id, new_id)
                    # Copy Datum der Wiederaufnahme in Erstgespräch und
                    # Beginn des CM
                    data['datum_cm_start'] = data['cm_wiederauf_genehm_datum_1']
                    data['erstgespraech'] = data['cm_wiederauf_genehm_datum_1']
                    # delete 'datum_cm_ende' as it must not be written
                    # in the new case
                    del data['datum_cm_ende']
                    data = _convert4Formed(data, new_id)
                    data['phase:%s' % new_id] = '2'
                    data['cm2c:%s' % new_id] = '1'

                    new_it.setData(data)
                    new_it.commit()

                    mark_case(new_id, 'WA')
                    create_logbook(id=new_id, header=_('Reopening'),
                                   text=_('For this case an accepted reopening '
                                           'of the integration process was '
                                           'documented.'))

                    ## Markiere alten Fall zur Anonymisierung
                    old_case = g.case_factory.loadById(id)
                    old_case.setState(4)
                    mark_case(old_case.id, 'WAU')
                    create_logbook(id=id, header=_('Reopening'),
                                   text=_('For this case a reopening of the '
                                           'integration process was done '
                                           'in a new case document.'
                                           ))
                else:


                    data['cm2c_t'] += "\n" + data['cm2c_t2']
                    del data['cm2c_t2']
                    save_formdata(id, data)
                    create_logbook(header=_('Extension reopening'),
                                   text=_('For this case an accepted extension '
                                           'of the reopening of the integration'
                                           ' process was documented.'))
            except Exception, err:
                log.exception(err.__dict__)
            if c.num_wiederaufnahme < 1:
                id = new_it.getRootNode().getIdentifier()
                c.success_for = _(u"Case document successfully reopened.")
                c.success_text = _(u'''\
The case document has been successfully reopened. Click on OK to open the
reopened case document.''')
                c.url_ok = h.url(controller="case",
                                     action="select",
                                     id="%s" % id)
            else:
                c.success_for = _(u"Case document successfully reopened.")
                c.success_text = _(u'''\
The case document has been successfully reopened. Click on OK to get to the
entry documentation of the case document.''')
                c.url_ok = h.url(controller="formularpage",
                                     action="select",
                                     id="page-0:%s" % id)
            return render('caselifetime/dialogs/success_reopen.mako')
        else:
            if c.num_wiederaufnahme < 1:
                c.confirm_for = _(u"Reopen case?")
                c.question = _(u"""\
Do you want to reopen the case? Then the current case document will be
anonymized and important parts of the case document copied from phase A and
phase B in the new case document. You will work with the new case document.""")
                c.url_yes = h.url(controller="caselifetime",
                                  action="reopen_ok",
                                  id=id, confirmed="1")
                c.url_no = h.url(controller="caselifetime", action="index",
                                 id=id)
            else:
                c.confirm_for = _(u"Do you want to reopen the case for 3 months?")
                c.question = _(u"""\
Do you want to reopen the case for 3 months? You can edit the case document
again.""")
                c.url_yes = h.url(controller="caselifetime",
                                  action="reopen_ok",
                                  id=id, confirmed="1")
                c.url_no = h.url(controller="caselifetime", action="index",
                                 id=id)
            return render('caselifetime/dialogs/confirm_reopen.mako')

    def reopen_notok(self, id, confirmed='0'):
        factory = InstanceFactory(g.formedTree, PostgresDBInterface())
        it = factory.loadInstanceTreeByIdentifier(session['case'].id)
        checkcase(it, id)
        data = {}
        if confirmed == '1':
            try:
                # Füge alte params aus dem Dialog in der verwaltung
                # hinzu und schreibe alles weg
                save_formdata(id, data)
                create_logbook(header=_('Reopening'),
                               text=_('For this case a not accepted reopening '
                                       'of the integration process '
                                       'was documented.'))
            except Exception, err:
                log.exception("")

            c.success_for = _(u"Not accepted reopening documented.")
            c.success_text = _(u'''\
The not accepted reopening was documented. Click on OK to get to the
entry documentation of the case document.''')
            c.url_ok = h.url(controller="formularpage",
                                 action="select",
                                 id="page-0:%s" % id)
            return render('caselifetime/dialogs/success_reopen.mako')
            return self.index()
        else:
            c.confirm_for = _(u"Do you want to document not accepted reopening?")
            c.question =  _(u"""\
You have entered, that no acceptence is available. \
If you continue, reopening, extension or addition will not be possible.""")
            c.url_yes = h.url(controller="caselifetime", action="reopen_notok",
                              id=id, confirmed="1")
            c.url_no = h.url(controller="caselifetime", action="index", id=id)
            return render('caselifetime/dialogs/confirm_reopen.mako')

    def reopenAction(self):
        factory = InstanceFactory(g.formedTree, PostgresDBInterface())
        it = factory.loadInstanceTreeByIdentifier(session['case'].id)
        case_id = it.getRootNode().getIdentifier()
        checkcase(it, case_id)
        S = _set_data
        form_result = {}
        form_errors = {}

        if c.num_wiederaufnahme < 1:
            if request.params.get('cm_wiederauf_genehm_1') == '0':
                validator = ReopenFormValidator1abgel()
            else:
                validator = ReopenFormValidator1()
        else:
            if request.params.get('cm_wiederauf_genehm_2') == '0':
                validator = ReopenFormValidator2abgel()
            else:
                validator = ReopenFormValidator2()

        try:
            form_result = validator.to_python(request.params)
            session['lifetimeparams'] = form_result
            session.save()
            try:
                # Lets try to insert some dates (do not actually insert
                # them) and check if they match all rules. Omit
                # datum_cm_ende from the rule check because it's only in
                # the form to make sure the new dates are later than
                # datum_cm_ende and trying to set it may trigger the
                # backdating rules.
                data_for_rule_check = form_result.copy()
                if data_for_rule_check.has_key("datum_cm_ende"):
                    del data_for_rule_check["datum_cm_ende"]
                S(it, data_for_rule_check, case_id, do_write=False)
                if (form_result.get('cm_wiederauf_genehm_1') == '1'
                    or form_result.get('cm_wiederauf_genehm_2') == '1'):
                    return self.reopen_ok(session['case'].id)
                else:
                    return self.reopen_notok(session['case'].id)
            except SetDataException, err:
                form_result_1, form_errors = toErrorDicts(err)
        except formencode.Invalid, error:
            #TODO: use htmlfill
            form_result = error.value
            form_errors = error.error_dict or {}

        form = render('/caselifetime/overview.mako')
        return formencode.htmlfill.render(form,
                                          defaults=form_result,
                                          errors=form_errors)

    def renewAction(self):
        factory = InstanceFactory(g.formedTree, PostgresDBInterface())
        it = factory.loadInstanceTreeByIdentifier(session['case'].id)
        case_id = it.getRootNode().getIdentifier()
        S = _set_data
        checkcase(it, case_id)

        validator = RenewFormValidator()
        form_result = {}
        form_errors = {}

        try:
            form_result = validator.to_python(request.params)
            session['lifetimeparams'] = form_result
            session.save()
            try:
                S(it, form_result, case_id, do_write=False)
                if form_result.get('cm_neuauf_genehm') == '1':
                    return self.renew_ok(case_id)
                else:
                    return self.renew_notok(case_id)
            except SetDataException, err:
                form_result_1, form_errors = toErrorDicts(err)

        except formencode.Invalid, error:
            form_result = error.value
            form_errors = error.error_dict or {}

        form_result = form_result
        form_errors = form_errors
        form = render('/caselifetime/overview.mako')
        return formencode.htmlfill.render(form,
                                          defaults=form_result,
                                          errors=form_errors)
