# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008, 2010 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
#

import logging

from mpulsweb.lib.base import session
from mpulsweb.lib.db import db

from mpulsweb.model.user import UserObject
from mpulsweb.model.agencysettings import Agency
from mpulsweb.model.statement import PrivacyStatement
from mpulsweb.model.case import MpulsCase, MpulsCaseFactory, \
     MpulsCaseOverview, MpulsCaseBundle, MpulsCaseState

from waskoweb.model.statement import DiscretionStatement

log = logging.getLogger(__name__)

# SQL STATEMENTS
INIT_CASE_SQL = """
UPDATE master_tbl_view
SET
    mitarb_zv = %(vorname)s,
    mitarb_zn = %(nachname)s,
    fkz = %(fkz)s,
    ks = %(agencyname)s
WHERE id = %(id)s"""

SET_EDITOR_SQL = """SELECT set_case_editor(%(userid)s, %(caseid)s)"""
SET_EDITOR_NAME_SQL = """
SELECT set_case_editor_name(%(firstname)s, %(lastname)s, %(caseid)s)"""


class CaseBundle(MpulsCaseBundle):

    pass


class CaseOverview(MpulsCaseOverview):

    pass


class State(MpulsCaseState):

    pass


class CaseFactory(MpulsCaseFactory):

    def _initNewCase(self, id):
        pass
        conn, cur = None, None
        try:
            user = session['USER_AUTHORIZED']
            agency = Agency()
            fields = {
                'vorname': user.first_name,
                'nachname': user.last_name,
                'fkz': agency.getFKZ(),
                'agencyname': agency.getName(),
                'id': int(id)
                }
            conn = db.getConnection()
            cur = conn.cursor()
            #print INIT_CASE_SQL % fields
            cur.execute(INIT_CASE_SQL, fields)
            conn.commit()
        finally:
            db.recycleConnection(conn, cur)


class Case(MpulsCase):

    def getHashFields(self):
        """Returns a dictionary of fields which is used to build the hashkey
        for the meta server"""
        fields = {}
        fields['firstname'] = self.get_value('vorname')
        fields['birthname'] = self.get_value('name')
        fields['gender'] = self.switch_gender_field(
                                       self.get_value('geschlecht'))
        fields['birthdate'] = self.get_value('geburtsdatum')
        return fields

    def switch_gender_field(self, gender):
        """switch_gender_field switches male/female number, because in Wasko
        these numbers are different as in base"""
        if gender == 0:
            return 1
        elif gender == 1:
            return 0
        return gender

    #def setEditor(self, userid):
    #    """Sets the name of the editor in formular of the case.  First call
    #    setEditor function in baseclass to transfer the case to the new user.
    #    After that set the name."""
    #    MpulsCase.setEditor(self, userid)
    #    con, cur = None, None
    #    con = db.getConnection()
    #    user = UserObject(userid)
    #    fields = {'caseid': self.id, 'firstname': user.first_name,
    #              'lastname': user.last_name}
    #    cur = con.cursor()
    #    try:
    #        cur.execute(SET_EDITOR_NAME_SQL, fields)
    #        con.commit()
    #    except Exception, e:
    #        log.debug(e)
    #        con.rollback()
    #    finally:
    #        db.recycleConnection(con, cur)
    #    return self.editor
    
    def getDiscretionStatement(self):
        self.discretion_statement = None
        """Returns the discretion statement of the case"""
        if not self.discretion_statement is None:
            return self.discretion_statement
        try:
            # Create privacy_statement
            self.discretion_statement = DiscretionStatement(self, Agency())
            return self.discretion_statement
        except:
            self.discretion_statement = None
            raise
        return None

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
