# -*- coding: utf-8 -*-

#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
#
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
# Sascha L. Teichmann <teichmann@intevation.de>
# Roland Geider <roland.geider@intevation.de>

import logging
import formencode

from pylons import request, tmpl_context as c, app_globals as g
from pylons.controllers.util import redirect

from mpulsweb.lib.translation import _

from mpulsweb.lib.db import db
from mpulsweb.lib.base import BaseController, render, session
from mpulsweb.lib.helpers import url
from mpulsweb.lib.security import checkRole, getDbName
from mpulsweb.lib.validators import ValidatorStateObject, NewUserGroupForm, \
    EditUserGroupForm , NewUserForm, EditUserForm, DeleteUserHelperForm
from mpulsweb.model.user import UserListObject, UserObject, UserGroupList, \
    UserGroup, UserGroupFactory
from mpulsweb.model.case import MpulsCaseOverview, MpulsCaseBundle
from mpulsweb.model.phase import PhaseFactory 
from mpulsweb.model.agency import MpulsAgencyOverview
from mpulsweb.controllers.case_overview import default_phases

log = logging.getLogger(__name__)


def user_has_cases(user):
    conn, cur = None, None
    fields = {'userid': user.id}
    sql = ("select count(id) from master_tbl_view"
           " where bearbeiter_id = %(userid)s")
    try:
        conn = db.getConnection()
        cur = conn.cursor()
        cur.execute(sql, fields)
        result = cur.fetchone()
        return int(result[0]) > 0
    except Exception, e:
        log.exception(e)
        return True
    finally:
        db.recycleConnection(conn, cur)


class AdministrationController(BaseController):

    @checkRole('admin')
    def index(self):
        return self.overviewUser()

    # USER
    @checkRole('admin')
    def overviewUser(self):
        list = UserListObject()
        c.user_overview = list.getUserList()
        return render('/administration/overview_user.mako')

    @checkRole('admin')
    def newUser(self):
        c.form_errors = {}
        c.form_result = {}
        c.uo = UserObject()
        
        # Load the agency list if application is running as evaluation server
        if g.mpuls_config.is_enabled('module', 'agency-overview'):
            agency_overview = MpulsAgencyOverview()
            agency_overview.search('sort_field:fkz;sort_order:asc')
            c.agency_list = agency_overview.getDatasets()
        
        return render('/administration/new_user.mako')

    @checkRole('admin')
    def newUserAction(self):
        validator = NewUserForm()
        c.user_password = None
        try:
            agency = getDbName()
            state = ValidatorStateObject()
            state.agency = agency
            c.form_result = validator.to_python(request.params, state)
            c.uo = UserObject()
            # Load the agency list if application is running as evaluation server
            if g.mpuls_config.is_enabled('module', 'agency-overview'):
                agency_overview = MpulsAgencyOverview()
                agency_overview.search('sort_field:fkz;sort_order:asc')
                c.agency_list = agency_overview.getDatasets()
            # Set initial data which is needed to create a new user
            login = c.form_result.get('login', '')
            role = c.form_result.get('role', '')
            c.uo.setData(c.form_result)
            # create user
            c.user_password = c.uo.createLogin(login, role, agency)
            return render('/administration/new_user_result.mako')
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            form = render('/administration/new_user.mako')
            return formencode.htmlfill.render(form,
                                              c.form_result, c.form_errors,
                                              auto_insert_errors=False)

    @checkRole('admin')
    def showUser(self, id=None):
        id = self._checkInt(id)
        c.uo = UserObject(id)
        c.vuo = UserGroup(c.uo.standin)
        # Load the agency list if application is running as evaluation server
        if g.mpuls_config.is_enabled('module', 'agency-overview'):
            agency_overview = MpulsAgencyOverview()
            agency_overview.search('sort_field:fkz;sort_order:asc')
            c.agency_list = agency_overview.getDatasets()
        return render('/administration/show_user.mako')

    @checkRole('admin')
    def editUser(self, id=None):
        id = self._checkInt(id)
        c.form_errors = {}
        c.form_result = {}
        c.uo = UserObject(id)
        # Load the agency list if application is running as evaluation server
        if g.mpuls_config.is_enabled('module', 'agency-overview'):
            agency_overview = MpulsAgencyOverview()
            agency_overview.search('sort_field:fkz;sort_order:asc')
            c.agency_list = agency_overview.getDatasets()
        return render('/administration/edit_user.mako')

    @checkRole('admin')
    def editUserAction(self):
        validator = EditUserForm()
        try:
            uid = request.params['uid']
            c.uo = UserObject(uid)
            # Load the agency list if application is running as evaluation server
            if g.mpuls_config.is_enabled('module', 'agency-overview'):
                agency_overview = MpulsAgencyOverview()
                agency_overview.search('sort_field:fkz;sort_order:asc')
                c.agency_list = agency_overview.getDatasets()
            form_result = validator.to_python(request.params)
            c.uo.setData(form_result)
            c.uo.store()
            c.dialog_title = _('User configs saved!')
            c.dialog_text = _("Please click okay to continue.")
            c.url_ok = url(controller='administration',
                               action='overviewUser')
            return render('/administration/dialogs/success_edit_user.mako')
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            return render('/administration/edit_user.mako')

    @checkRole('admin')
    def resetUserPassword(self, id=None, confirmed='0'):
        id = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        c.uo = UserObject(id)
        if confirmed:
            c.new_password = c.uo.resetPassword()
            return render('/administration/reset_password.mako')
        else:
            c.context = "../main.mako"
            c.dialog_title = _('Reset user password?')
            c.dialog_text = "%s %s, %s" % (_('Confirm reset user password:'),
                                           c.uo.last_name, c.uo.first_name)

            c.url_yes = url(controller='administration',
                                action='resetUserPassword', id=id, confirmed=1)
            c.url_no = url(controller='administration',
                               action='showUser', id=id)
            return render('/administration/dialogs/confirm_resetpassword.mako')

    # Old deleteUser method, which does not raise a helper dialog to transfer
    # remaining cases to a another user before deletion. Simply raise an
    # error-dialog an cance action.

    #@checkRole('admin')
    #def deleteUser(self, id, confirmed='0'):
    #    id = self._checkInt(id)
    #    confirmed = self._checkBool(confirmed)
    #    uo = UserObject(id)

    #    # Make sure that the user can not delete himself
    #    if uo.id == session.get('USER_AUTHORIZED').id:
    #        c.dialog_title = _('User not deleted!')
    #        c.dialog_text = _('Not possible to delete current login user. '
    #                          'Click on okay to continue.')
    #        c.url_ok = url(controller='administration',
    #                           action='overviewUser')
    #        return render('/administration/dialogs/failed_delete_user.mako')

    #    if confirmed:
    #        # Check if there are cases for which the user is the editor. If
    #        # he is an editor then raise a error dialog
    #        if user_has_cases(uo):
    #            c.dialog_title = _(u'The user can not be deleted')
    #            c.dialog_text = _(u'The user can not be deleted, because he '
    #                              u'is agent of at least one case.'
    #    u' Please move all cases of this agent to a different agent and repeat'
    #    u' the action. Click on \"OK\" to navigate back to the user overview.')
    #            c.url_ok = url(controller='administration',
    #                               action='overviewUser')
    #            return render('/administration/dialogs/failed_delete_user.mako')
    #        else:
    #            uo.delete()
    #            c.dialog_title =  _('User deleted!')
    #            c.dialog_text = _('User sucessfully deleted. '
    #                              'Click okay to continue.')
    #            c.url_ok = url(controller='administration',
    #                               action='overviewUser')
    #            return render('/administration/dialogs/success_delete_user.mako')

    #    else:
    #        c.context = "../main.mako"
    #        c.dialog_title = _('Delete user?')
    #        c.dialog_text = _('Please confirm to delete user '
    #          '<strong>%s, %s</strong> (login: %s).') % (uo.last_name,
    #                                                     uo.first_name,
    #                                                     uo.login)
    #        c.url_yes = url(controller='administration',
    #                            action='deleteUser', id=id, confirmed=1)
    #        c.url_no = url(controller='administration',
    #                           action='overviewUser')
    #        return render('/administration/dialogs/confirm_deleteuser.mako')

    @checkRole('admin')
    def deleteUser(self, id, confirmed='0'):
        id = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        uo = UserObject(id)
        # Make sure that the user can not delete himself
        if uo.id == session.get('USER_AUTHORIZED').id:
            c.failed_for = _('User not deleted!')
            c.failed_text = _('Not possible to delete current login user.'
                    ' Click on okay to continue.')
            c.url_ok = url(controller='administration',
                               action='overviewUser')
            return render('/administration/dialogs/failed_delete_user.mako')

        if confirmed:
            # Check if there are cases for which the user is the editor. If
            # he is an editor then raise a dialog which allows the user to
            # transfer the cases.
            overview = MpulsCaseOverview()
            phases = []
            for p in default_phases():
                phases.append('phase:%s' % p)
            overview.search('editor:%s;'
                            'state:1;state:2;state:3;state:4;state:5;%s'
                            % (uo.id, ";".join(phases)))
            if overview.getDatasets():
                list = UserListObject()
                filter = [u.id for u in list.getAdminList()]
                filter.append(uo.id) #the user we want to delete
                c.user_list = list.getUserList(filter)
                c.cases = overview
                c.phases = PhaseFactory().build() 
                log.debug(c.cases)
                c.delete_user_id = id
                return render('/administration/delete_user_helper.mako')
            else:
                self._before_delete_user(id, confirmed='0')
                uo.delete()
                c.success_for = _('User deleted!')
                c.success_text = _('User sucessfully deleted. '
                        'Click okay to continue.')
                c.url_ok = url(controller='administration',
                                   action='overviewUser')
                return render('/administration/dialogs/'
                              'success_delete_user.mako')

        else:
            c.delete_user_id = id
            c.context = "../main.mako"
            c.confirm_for = _('Delete user?')
            c.question = _('Please confirm to delete user '
              '<strong>%s, %s</strong> (login: %s).') % (uo.last_name,
                                                         uo.first_name,
                                                         uo.login)
            c.url_yes = url(controller='administration',
                                action='deleteUser', id=id, confirmed=1)
            c.url_no = url(controller='administration',
                               action='overviewUser')
            return render('/administration/dialogs/confirm_deleteuser.mako')


    def _before_delete_user(self, id, confirmed='0'):
        """
        Application specific actions to perform before actually deleting the user.
        For example, applications with a structure part can remove the rights to
        the user to access it.
        """
        pass

    @checkRole('admin')
    def deleteUserHelper(self):
        validator = DeleteUserHelperForm()
        all_cases = MpulsCaseOverview()
        c.notification_for = _('Assign case document')
        c.form_errors = {}
        c.form_result = {}

        if 'editor' in request.params:
            try:
                data = formencode.variabledecode.variable_decode(request.params)
                form_result = validator.to_python(data)
                case_bundle = MpulsCaseBundle(form_result.get('case_id', []))
                new_editor = UserObject(form_result.get('editor'))
                del_user = UserObject(form_result.get('user_id'))
                all_cases.search('editor:%s;state:1;state:2;state:3;state:4;'
                                 'state:5' % del_user.id)
                num = case_bundle.setEditor(form_result.get('editor'))
                rest = all_cases.numDatasets() - case_bundle.numDatasets()
                c.url_ok = url(controller='administration',
                                   action='deleteUser',
                                   id=form_result.get('user_id'), confirmed=1)
                c.success_for = _('Case document assigned!')
                if rest > 0:
                    c.success_for = _('Case document assigned. '
                            'More case documents exist!')
                    c.success_text = (_(u'Case document assigned to %s, %s.'
                        u' %s, %s has more case documents.'
                        u' Please click on OK to continue.')
                                      % (new_editor.first_name,
                                         new_editor.last_name,
                                         del_user.first_name,
                                         del_user.last_name))
                else:
                    c.success_for = _('Case document assigned!')
                    c.success_text = (_('Case document assigned to %s, %s.'
                        ' Please click on OK to delete %s, %s.')
                                      % (new_editor.first_name,
                                         new_editor.last_name,
                                         del_user.first_name,
                                         del_user.last_name))
                return render('/administration/dialogs/'
                              'notificate_delete_user_helper.mako')
            except formencode.Invalid, error:
                return self.deleteUser(form_result.get('user_id'), '1')
        else:
            redirect(url(controller="administration", action='overviewUser'))

    # USERGROUPS
    @checkRole('admin')
    def overviewUserGroups(self):
        # Get gid of usergroups so we can filter them out.
        user = UserListObject()
        filter = [u.gid for u in user.getUserList()]

        list = UserGroupList()
        c.group_overview = list.getGroupList(filter)
        return render('/administration/overview_usergroups.mako')

    @checkRole('admin')
    def newUserGroup(self):
        c.form_errors = {}
        c.form_result = {}
        return render('/administration/new_usergroup.mako')

    @checkRole('admin')
    def newUserGroupAction(self):
        factory = UserGroupFactory()
        validator = NewUserGroupForm()
        try:
            c.form_result = validator.to_python(request.params)
            c.ugo = factory.create(c.form_result.get('name'))
            c.dialog_title = _('Create user group!')
            c.dialog_text =  _(u'User group successfully created. '
                    u'Please click on OK to continue to the'
                    u' user group overview')
            c.url_ok = url(controller='administration',
                               action='overviewUserGroups')
            return render('/administration/dialogs/success_new_usergroup.mako')
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            form = render('/administration/new_usergroup.mako')
            return formencode.htmlfill.render(form,
                                              c.form_result,
                                              c.form_errors,
                                              auto_insert_errors=False)
        except Exception, e:
            log.exception(e)
            c.dialog_title = _('Failed to create user group!')
            c.dialog_text =  _('Failed to create the user group.'
  ' Please click on OK to continue in the user overview.')
            c.url_ok = url(controller='administration',
                               action='overviewUserGroups')
            return render('/administration/dialogs/failed_new_usergroup.mako')

    @checkRole('admin')
    def showUserGroup(self, id=None):
        factory = UserGroupFactory()
        id = self._checkInt(id)
        group = factory.load(id)
        groupmember = group.getUser()
        c.group = group
        c.groupmember = groupmember
        return render('/administration/show_usergroup.mako')

    @checkRole('admin')
    def editUserGroup(self, id=None):
        id = self._checkInt(id)
        factory = UserGroupFactory()
        group = factory.load(id)
        memberlist = group.getUser().getUserList()
        c.form_errors = {}
        c.form_result = {'name': group.name,
                         'id': group.id,
                         'standin': [user.id for user in memberlist]}
        c.group = group
        userlist = UserListObject()
        admins = userlist.getAdminList()
        c.userlist = userlist.getUserList(filter=[u.id for u in admins])
        form = render('/administration/edit_usergroup.mako')
        return formencode.htmlfill.render(form,
                                          c.form_result, c.form_errors,
                                          auto_insert_errors=False)

    @checkRole('admin')
    def editUserGroupAction(self):
        validator = EditUserGroupForm()
        factory = UserGroupFactory()
        userlist = UserListObject()
        admins = userlist.getAdminList()
        c.userlist = userlist.getUserList(filter=[u.id for u in admins])

        try:
            state = ValidatorStateObject()
            data = formencode.variabledecode.variable_decode(request.params)
            state.groupid = data.get('id')
            form_result = validator.to_python(data, state)
            group = factory.load(form_result.get('id'))
            group.setData(form_result)
            group.setUser(form_result.get('standin'))
            group.store()
            c.group = group
            c.dialog_title = _('Edit user group!')
            c.dialog_text = _('The user group was successfully edited. '
                    'Please click on OK '
                    'to continue in the user group overview.')
            c.url_ok = url(controller='administration',
                               action='overviewUserGroups')
            return render('/administration/dialogs/success_edit_usergroup.mako')
        except formencode.Invalid, error:
            c.form_result = error.value
            c.group = factory.load(c.form_result.get('id'))
            c.form_errors = error.error_dict or {}
            form = render('/administration/edit_usergroup.mako')
            return formencode.htmlfill.render(form,
                                              c.form_result, c.form_errors,
                                              auto_insert_errors=False)

    @checkRole('admin')
    def deleteUserGroup(self, id, confirmed='0'):
        id = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        factory = UserGroupFactory()
        try:
            ugo = factory.load(id)
            if confirmed:
                ugo.delete()
                c.dialog_title = _('User group deleted!')
                c.dialog_text =  _('The user group was successfully deleted.'
    ' Please click on OK to continue in the user group overview.')
                c.url_ok = url(controller='administration',
                                   action='overviewUserGroups')
                return render('/administration/dialogs/'
                              'success_deletegroup_user.mako')
            else:
                c.context = "../main.mako"
                c.dialog_title = _('Delete user group?')
                c.dialog_text =  _('Do you want to delete the user group and '
                        'lose the data?')
                c.url_yes = url(controller='administration',
                                    action='deleteUserGroup', id=ugo.id,
                                    confirmed=1)
                c.url_no = url(controller='administration',
                                   action='overviewUserGroups')
                return render('/administration/dialogs/'
                              'confirm_deleteusergroup.mako')
        except Exception, e:
            log.exception(e)
            c.dialog_title =_('Failed to delete the user group!') 
            c.dialog_text = _('Deletion of the user group failed. '
                    'Please click on OK to continue'
                    ' in the user group overview.')
            c.url_ok = url(controller='administration',
                               action='overviewUserGroups')
            return render('/administration/dialogs/failed_new_usergroup.mako')


# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
