# -*- coding: utf-8 -*-

import logging
import formencode

import paste

from pylons import request, tmpl_context as c, url
from pylons.controllers.util import redirect
from mpulsweb.lib.translation import _, ungettext

from mpulsweb.lib.base import BaseController, render, session, g
from mpulsweb.lib.validators import UploadCaseDocumentForm, \
                                    UploadCaseDocumentWDescForm, \
                                    RenameDocumentForm, \
                                    RenameDocumentWDescForm

from mpulsweb.lib.security import checkRole
import mpulsweb.lib.helpers as h

from mpulsweb.model.document import Document, deleteDocument, renameDocument, \
                                    listDocuments


log = logging.getLogger(__name__)

class CasedocumentController(BaseController):

    @checkRole(('admin', 'cm'))
    def index(self, id):
        return self.overview(id)

    @checkRole(('admin', 'cm'))
    def overview(self, id):
        id = self._checkInt(id)
        case = self._loadCase(id, read_only=True)
        c.files = case.getDocuments()
        return render('/documents/case_overview.mako')
   
    def create_document_list(self, files):
        """Creates a list of the documents, containing (id,name, description).
        So in renameDocument this data can be accessed. """
        document_list = []
        for doc in files:
            document_list.append((doc.id, doc.name, doc.description))
        return document_list

    @checkRole('cm')
    def new(self):
        return render('/documents/case_new.mako')

    @checkRole('cm')
    def newAction(self):
        if g.mpuls_config.is_enabled("module", "documents-description"):
            validator = UploadCaseDocumentWDescForm()
        else:
            validator = UploadCaseDocumentForm()
        try:
            result = validator.to_python(request.params)
            doc = Document()
            filename = result['file']['filename']
            content = result['file']['content']
            case = result['case']
            if result['name']:
                # Make sure the new name has a (3 letter) extension
                if '.' not in result['name'][-4:]:
                    extension = filename.rpartition('.')[2]
                    filename = result['name'] + '.' + extension
                else:
                    filename = result['name']
            if g.mpuls_config.is_enabled("module", "documents-description"):
                doc.create(filename, content, case, 
                           description=result['description'])
            else:
                doc.create(filename, content, case)
            c.dialog_title = _(u"Attachment added!")
            c.dialog_text = _(u"Please click OK to continue.")
            c.url_ok = h.url(controller="casedocument", action="index",
                                 id=case)
            return render('/documents/dialogs/success_attachment.mako')
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            log.debug(c.form_result)
            log.warn("Invalid form data for case document %s" % c.form_errors)
            form = render('/documents/case_new.mako')
            return formencode.htmlfill.render(form,
                                              defaults=c.form_result,
                                              auto_insert_errors=False,
                                              errors=c.form_errors)

    @checkRole(('admin', 'cm'))
    def show(self, id):
        id = self._checkInt(id)
        try:
            document = Document(id)
            document.load(True)
            data = paste.fileapp.DataApp(document.data,
                                         content_type=document.mime,
                                         cache_control='must-revalidate',
                                         pragma='public')
            return data(request.environ, self.start_response)
        except Exception, e:
            log.exception(e)
            redirect(url(controller="case", action="documents"))

    @checkRole('cm')
    def delete(self, id, confirmed):
        id = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        if confirmed == 1:
            try:
                deleteDocument(id, True)
            except Exception, e:
                log.exception(e)
            c.url_ok = h.url(controller="casedocument", action="index" ,
                                 id=session['case'].id)
            c.success_for = _(u"Attachment deleted!")
            c.success_text = _(u"Please click OK to continue.")
            return render('/casemanagement/dialogs/'
                          'success_delete_attachment.mako')
        else:
            docs = listDocuments(session['case'].id)
            for doc in docs:
                if int(doc.id) == id:
                    name = doc.getName()
                    description = doc.description
                    break
            if not name:
                name = ""
            if not description:
                description = ""

            c.context = "../main.mako"
            c.confirm_for = _(u"Delete attachment?")
            c.question = _(u"Do you want to irrevocably delete the attachment '%s'?") % name
            c.url_yes = h.url(controller='casedocument', action='delete',
                                  id=id, confirmed=1)
            c.url_no = h.url(controller="casedocument", action="index",
                                 id=session['case'].id)
            return render('/casemanagement/dialogs/'
                          'confirm_delete_attachment.mako')
    @checkRole('cm')
    def rename(self, id, confirmed='0'):
        c.id = self._checkInt(id)
        docs = listDocuments(session['case'].id)
        for doc in docs:
            if int(doc.id) == c.id:
                c.name = doc.getName()
                c.description = doc.description
                break
        if not c.name:
            c.name = ""
        if not c.description:
            c.description = ""

        return render('/casemanagement/rename_attachment.mako')

    @checkRole('cm')
    def renameAction(self, id):
        if g.mpuls_config.is_enabled("module", "documents-description"):
            validator = RenameDocumentWDescForm()
        else:    
            validator = RenameDocumentForm()
        try:
            result =  validator.to_python(request.params)
            if result.has_key('name'):
                filename = result['name']
            else:
                filename = ""
            if result.has_key('description'):
                desc  = result['description'] 
            else:
                desc = ""
            renameDocument(id, filename, desc, True)
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            log.debug(c.form_result)
            form = render('/casemanagement/rename_attachment.mako')
            return formencode.htmlfill.render(form,
                                              defaults=c.form_result,
                                              auto_insert_errors=False,
                                              errors=c.form_errors)
        return self.overview(session['case'].id)
