# -*- coding: utf-8 -*-
import logging
import datetime

from pylons import app_globals as g, tmpl_context as c
from pylons.controllers.util import abort
from mpulsweb.lib.base import BaseController, render
from mpulsweb.lib.security import checkRole
from mpulsweb.lib.db import db


log = logging.getLogger(__name__)

SQL_FKZ = """SELECT fkz, job_percentage FROM fkz_tbl;"""

class EvaljobpercentageController(BaseController):

    '''Display the jobpercentages on the evaluation server'''

    @checkRole(('cm','admin'))
    def index(self):
        if g.mpuls_config.is_enabled('job_percentage', 'showeval'):
            c.fkz_list = self._load_fkz_tbl()
            c.datadump = self._get_datadump()
            return render("/jobpercentage/evaljobpercentage.mako")
        else:
            abort(404)

    def _load_fkz_tbl(self):
        con, cur = None,None
        fkz_list = []

        try:
            con = db.getConnection()
            cur = con.cursor()
            cur.execute(SQL_FKZ)

            for row in cur.fetchall():
                fkz_list.append((row[0], row[1]))
        except:
            log.exception("load_fkz_failed")
        finally:
            db.recycleConnection(con, cur)

        return fkz_list

    def _get_datadump(self):
        v_dataset = None
        try:
            try:
                conn = db.getConnection()
                cur = conn.cursor()
                cur.execute("SELECT value FROM dbinfo WHERE key = 'datadump'")
                datestr = cur.fetchone()[0]
                y,m,d = datestr.split('-')
                v_dataset = datetime.date(int(y), int(m), int(d))
            except:
                log.exception('Error: get datadump failed')
        finally:
            db.recycleConnection(conn, None)
        return v_dataset

