# -*- coding: utf-8 -*-
import logging

from pylons import tmpl_context as c, app_globals as g

from mpulsweb.lib.translation import _
from mpulsweb.lib.base import BaseController, h, render, request
from mpulsweb.lib.security import checkRole
from mpulsweb.model.phase import PhaseFactory, PC_FULLAUTO
from mpulsweb.model.meta import get_meta_client, MetaException

log = logging.getLogger(__name__)

class PhaseController(BaseController):

    @checkRole(('admin', 'cm'))
    def index(self, id):
        return self.overview(id)

    @checkRole(('admin', 'cm'))
    def overview(self, id):
        id = self._checkInt(id)
        c.phases = PhaseFactory().load(id)
        # Only show buttons to set the phase if 
        c.show_actions = g.mpuls_config.get('phases', 'changemode') != PC_FULLAUTO 
        return render('/phase/phase.mako')

    @checkRole(('cm'))
    def setPhase(self, id, value):
        case = self._loadCase(id)
        # Set phase
        case.setPhase(value)

        # After setting the phase, transmit reportquota.
        if g.mpuls_config.is_enabled('module', 'meta-transmit-reportquota'):
            get_meta_client().transmit_reportquota()

        # After setting the phase. Show print-dialog
        c.show_meta_case_hint = (g.mpuls_config.is_enabled('case-module',
                                                           'meta-server')
                                 and case.getMeta().is_linked())
        if c.show_meta_case_hint:
            try:
                case.getMeta().push()
                c.show_meta_case_upload_hint =_('The data was sucessfully '
                                                'uploaded to the meta case.')
            except MetaException, exc:
                c.show_meta_case_upload_hint = _('Error! Data not uploaded.')
                log.exception("Meta data not automatic uploaded")
        c.dialog_title = _("Phase changed!")
        if c.show_meta_case_hint:
            c.dialog_text = _("The phase could be changed successfully and the"
                              " case has been synced with the Meta-Server."
                              " Please follow the shown steps to reach the phase"
                              " overview.")
        else:
            c.dialog_text = _("The phase could be changed successfully."
                              " Please follow the shown steps to reach the phase"
                              " overview.")
        c.url_ok = h.url(controller='phase', action='overview', id=case.id)
        return render('/phase/dialogs/success_set_phase.mako')

    @checkRole(('cm'))
    def editAction(self):
        case = self._loadCase(request.params.get('case_id'))
        newphase = None

        # TODO: This is ugly. The id of the phase to be set is submitted as key
        # (intvalue)) of the submitbutton. So to find the right value, try to
        # convert all keys to int, which should only work for the
        # submitbuttons.
        for newphase in request.params.iterkeys():
            try:
                newphase = int(newphase)
                break
            except:
                pass # convert failed, seems not to be the submit button.
                     # Continue...
        return self.setPhase(request.params.get('case_id'), newphase)
