# -*- coding: utf-8 -*-

import logging

from pylons import tmpl_context as c
from mpulsweb.lib.translation import _
from pylons.controllers.util import abort

from mpulsweb.lib.base import BaseController, render, session
import mpulsweb.lib.helpers as h
from mpulsweb.lib.navigation import get_navigation
from mpulsweb.controllers.formularpage import FormularpageController

log = logging.getLogger(__name__)


class RepeatgroupController(BaseController):

    def list(self, id):
        """List rg-pages as overview page"""
        pass

    def select(self, id):
        """Switch between rg in embedded rg"""
        # Currently this function is not needed as we do not support
        # navigationable repeatgroups
        return FormularpageController().select(session.get('navigation'
                                                           '.selectedpage'))

    def delete(self, id, confirmed='0'):
        """Delete rg"""
        log.debug("deleting repeat group %s" % id)
        if confirmed == '1':
            case = self._load_current_case()
            instance_tree = case.get_formed_instance_tree()
            rg = instance_tree.getItem(id)
            if not rg:
                abort(404, _('Repeat group not found.'))
            try:
                rg.getParent().deleteInstance(rg)
            except:
                log.exception("Error trying to delete repeat group %r:", id)
                c.status_msg = h.status_message(u"Fehler beim Löschen"
                                                u" des Datensatzes.")
            else:
                c.status_msg = h.status_message(u"Datensatz wurde erfolgreich"
                                                u" gelöscht.")

            # Build new Navigation
            navigation = get_navigation(case.id,
                                        session['navigation.openfolders'])
            session['navigation.tree'] = navigation
            session.save()
            return FormularpageController().select(session.get('navigation'
                                                               '.selectedpage'))
        else:
            c.confirm_for = _(u"Delete dataset?")
            c.question = _(u"Do you want to delete this dataset and\n"
                           u"irrevocably lose the data?")
            c.url_yes = h.url(controller='repeatgroup', action='delete',
                                  id=id, confirmed=1)
            c.url_no = h.url(controller='formularpage', action='select',
                                 id=session['navigation.selectedpage'])
            return render('/repeatgroup/dialogs/confirm_delete_rg_entry.mako')

    def new(self, id):
        """New rg"""
        log.debug("creating new repeat group %s" % id)
        case = self._load_current_case()
        instance_tree = case.get_formed_instance_tree()
        rgcontainer = instance_tree.getItem(id)
        if not rgcontainer:
            abort(404, _('repeat group container not found.'))
        rgcontainer.createInstance()

        # Build new Navigation
        navigation = get_navigation(session['case'].id,
                                    session['navigation.openfolders'])
        session['navigation.tree'] = navigation
        session.save()
        return FormularpageController().select(session.get('navigation'
                                                           '.selectedpage'))
