import os
import logging
from gettext import NullTranslations, translation

import pylons
from pylons.i18n.translation import get_lang, LanguageError

# Copied from pylons.i18n.translation. Adapted code to allow otional locations
# of language files

log = logging.getLogger(__name__)

def N_(value):
    """Null translation. Used to mark strings for later translation. Simply
    returns untranslated string"""
    return value

def ugettext(value):
    """Mark a string for translation. Returns the localized unicode
    string of value.

    Mark a string to be localized as follows::

        _('This should be in lots of languages')
    
    """
    return _ugettext(get_lang(), value)

_ = ugettext

def ungettext(singular, plural, n):
    """Mark a string for translation. Returns the localized unicode
    string of the pluralized value.

    This does a plural-forms lookup of a message id. ``singular`` is
    used as the message id for purposes of lookup in the catalog, while
    ``n`` is used to determine which plural form to use. The returned
    message is a Unicode string.

    Mark a string to be localized as follows::

        ungettext('There is %(num)d file here', 'There are %(num)d files here',
                  n) % {'num': n}

    """
    return _ungettext(get_lang(), singular, plural, n) 

def _ugettext(lang, value):
    dirs = []
    domains = []
    if pylons.config.get('mpuls.app.path.i18n'):
        domains.append(pylons.config.get('mpuls.app.instance'))
        dirs.append(pylons.config.get('mpuls.app.path.i18n'))

    # Adding defaults
    conf = pylons.config.current_conf()
    dirs.append(os.path.join(conf['pylons.paths']['root'], 'i18n'))
    domains.append('mpulsweb')

    for do in domains:
        for di in dirs:
            translator = _get_translator(lang, do, di)
            if translator is not None: 
                translated = translator.ugettext(value)
                if translated != value:
                    # Seems to be translated
                    return translated
    return value

def _ungettext(lang, singular, plural, n):
    dirs = []
    domains = []
    if pylons.config.get('mpuls.app.path.i18n'):
        domains.append(pylons.config.get('mpuls.app.instance'))
        dirs.append(pylons.config.get('mpuls.app.path.i18n'))

    # Adding defaults
    conf = pylons.config.current_conf()
    dirs.append(os.path.join(conf['pylons.paths']['root'], 'i18n'))
    domains.append('mpulsweb')

    for do in domains:
        for di in dirs:
            translator = _get_translator(lang, do, di)
            if translator is not None:  
                translated = translator.ungettext(singular, plural, n)
                # handle 0 as plural 
                if n > 1 or n == 0:
                    if translated != plural:
                        # Seems to be translated
                        return translated
                else:

                    if translated != singular:
                        # Seems to be translated
                        return translated
    return translated

def _get_translator(lang, domain='mpulsweb', dir=None, **kwargs):
    """Utility method to get a valid translator object from a language
    name"""
    #log.debug('getting translator for %s' % domain)
    if not lang:
        return NullTranslations()

    if not isinstance(lang, list):
        lang = [lang]

    if not dir:
        conf = pylons.config.current_conf()
        dir = os.path.join(conf['pylons.paths']['root'], 'i18n')

    try:
        # TODO: Do not build a new newbranslator every time. Save it in app_globals
        # e.g
        translator = translation(domain,dir,languages=lang, **kwargs)
        translator.pylons_lang = lang
        return translator
    except IOError, ioe:
        #log.debug('Can not load %s in %s' % (domain, dir))
        return None

def set_lang(lang):
    """Set the current language used for translations.

    ``lang`` should be a string or a list of strings. If a list of
    strings, the first language is set as the main and the subsequent
    languages are added as fallbacks.
    """
    translator = _get_translator(lang)
    environ = pylons.request.environ
    environ['pylons.pylons'].translator = translator
    if 'paste.registry' in environ:
        environ['paste.registry'].replace(pylons.translator, translator)
