# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008, 2010 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
# Authors:
# Roland Geider <roland.geider@intevation.de>
#

from mpulsweb.lib.db import db
from mpulsweb.model.agencysettings import Agency
from pylons import app_globals as g

import mpulsweb.model.phase as phase_mod


import datetime
import logging
log = logging.getLogger(__name__)

COUNT_ALL_CASES = """
SELECT DISTINCT COUNT(id) FROM master_tbl_view m
WHERE %(where)s"""

WHERE = """(%(start_date_column)s <= %(end_date)s AND %(start_date)s <= coalesce(%(end_date_column)s, now()) AND phase IN (%(successors)s)) """

class JobPercentage:

    def calulate_percentage(self, start=None, end=None):
        """Calculate all cases in phases divides thru job parts"""
        if start is None:
            start = "NOW()"
        if end is None:
            end = "NOW()"
        phases_from_factory = phase_mod.PhaseFactory().build()
        phases =  g.mpuls_config.get('job_percentage', 'phases')
        where = "TRUE"
        log.debug("JobPercentage: phases %s" % phases)
        if phases:
            phases = phases.split(',')
            where_buf = []
            for p in phases_from_factory:
                if p.id in phases:
                    succs = set()
                    fields = {'start_date_column': 'Now()',
                              'start_date': start,
                              'end_date': end,
                              'end_date_column': 'Now()'}
                    if p.start.datefield:
                        fields['start_date_column'] = p.start.datefield
                    if p.end.datefield:
                        fields['end_date_column'] = p.end.datefield
                    succs = phase_mod.get_phase_successors(p.id,
                                                  phases_from_factory)
                    fields["successors"] = ",".join(succs)
                    where_buf.append(WHERE % fields)
            if where_buf:
                where = " OR ".join(where_buf)

        conn = db.getConnection()
        cur = conn.cursor()
        fields = {'where': where}
        sql_query = COUNT_ALL_CASES % fields
        log.debug(sql_query)
        cur.execute(sql_query)
        cases = cur.fetchone()[0]

        agency = Agency()
        job_percentage = float(agency.getJobPercentage())
        log.debug("JobPercentage: cases %d, jobparts %f" %
                      (cases, job_percentage))
        if job_percentage and cases:
            # cases = # of cases
            # stellenanteile (job percentage) of the agency
            percentage = cases / float(job_percentage)

            return (cases, percentage)
        else:
            return (0, 0)
