## -*- coding: utf-8 -*-
<%!

from pylons import app_globals as g
import mpulsweb.lib.helper.filters as F
import mpulsweb.lib.helpers as h
from mpulsweb.lib.translation import _

all_roles = ['admin', 'cm']

## Fields to show in the case list.  Each entry is a tuple of the form
##     (TITLE, FIELDNAME, STYLE, ROLES, FORMATTER)
## Here, TITLE is the title of the field, FIELDNAME is the field name
## used in the search URL that's built for each column, STYLE is a CSS
## class name to use in the style attribute of the HTML TH-element,
## ROLES is a list with the names of the roles for which the field
## should be shown.  Finally, FORMATTER is a python callable that is
## called with the case object as argument and which should return the
## formatted value for the column.

fields = []
for d in g.case.get_overview_fields():
  f = d.name
  fn = d.overview_label
  fs = d.overview_style
  field = (fn,f,fs, all_roles,
           lambda case, d=d: h.shorten(h.format_object(case.get_value(d.name)), d.overview_length))
  fields.append(field)

# Extend fields with default fields in overview
fields.extend([
    (_('Editor'), "bearbeiter_id", "table_name",
     ["admin"],
     lambda case: "%s, %s" % (F.NA(case.getEditor().last_name),
                              F.NA(case.getEditor().first_name[:1]))),

    (_('Updated'), "zugriff", "table_name", ["cm"],
     lambda case: h.format_date(case.state.getAccessTime())),
  ])
%>

<%def name="build_list_headers()">
  <%
    # Normally CASE_OVERVIEW_SEARCHOPTIONS should contain at least the default
    # ordering fields when showing the case-overview (Set in
    # case_overview-controller). But as this template is included in other
    # use-cases too this is not always true. In theses cases be more error
    # tolerant and fallback to no (system default = id) sorting,
    # which should be ok in theses cases.
    s = session.get('CASE_OVERVIEW_SEARCHOPTIONS', {})
    sort_field = s.get('sort_field')
    sort_order = s.get('sort_order')
  %>
  % if g.mpuls_config.is_enabled('case-module', 'casebundle'):
    <th class="table_header_h table_checkbox">&nbsp;</th>
  % endif
  % for title, urlpart, style, roles, get_value in fields:
    % if h.hasRole(roles):
      <th class="${style} table_header_h">${title}
        <a href="${h.url(controller='case_overview', action='overview',
                             sort_field=urlpart, sort_order='asc')}">

            <img class   = "sorticon"
            %if sort_field == urlpart and sort_order == 'asc':
                src    = "${h.url('/images/icons/sort_asc_active.png')}"
                alt    = "${_("Sorted by %s in ascending order") % title}"
                title  = "${_("Sorted by %s in ascending order") % title}"
            %else:
                src    = "${h.url('/images/icons/sort_asc.png')}"
                alt    = "${_("Sort in ascending order")}"
                title  = "${_("Sort in ascending order")}"
            %endif
                width  = "10"
                height = "9"></a>
        <a href="${h.url(controller='case_overview', action='overview',
                             sort_field=urlpart, sort_order='desc')}">
            <img class   = "sorticon"
            %if sort_field == urlpart and sort_order == 'desc':
                src    = "${h.url('/images/icons/sort_desc_active.png')}"
                alt    = "${_("Sorted by %s in descending order") % title}"
                title  = "${_("Sorted by %s in descending order") % title}"
            %else:
                src    = "${h.url('/images/icons/sort_desc.png')}"
                alt    = "${_("Sort in descending order")}"
                title  = "${_("Sort in descending order")}"
            %endif
                width  = "10"
                height = "9"></a>
      </th>
    % endif
  % endfor
  <th class="table_status table_header_h">
    ${_('Status')}
  </th>
  <th class="table_action table_header_h">
    ${_('Actions')}
  </th>
</%def>

<%def name="build_list_row(case)">
  % if g.mpuls_config.is_enabled('case-module', 'casebundle'):
    <td><input type="checkbox" name="case_id" value="${case.id}"></td>
  % endif
  % for title, urlpart, style, roles, get_value in fields:
    % if h.hasRole(roles):
      % if case.getState().getState() != 5 and case.getState().getState() != 3:
        
        <td><a href="${h.url(controller='case', action='select',
                                 id=case.id, confirmed=0)}">${get_value(case) | F.NA}</a></td>
      % else:
        <td>${get_value(case) | F.NA}</td>
      % endif
    % endif
  % endfor
  <td>
    ${get_phase_icon(case)}
    % if g.mpuls_config.is_enabled('case-module', 'tags'):
        ${get_validity_icon(case)}
    % endif
    ${get_state_icon(case)}
    ${get_sync_icon(case)}
  </td>
  <td>
    ${get_actions(case)}
  </td>
</%def>

<%def name="get_validity_icon(case)">
  % for validity in g.mpuls_config.get('validities', 'enabled'):
    % if validity['id'] == case.getValidity():
      ##<a href="${h.url(controller='tags', action='overview', id=case.id)}">
        <img src="${h.url('/images/icons/%s' % validity['icon'])}" alt="${validity['name']}" title="${validity['name']}" width="22" height="22">
      ##</a>
    % endif
  % endfor
</%def>

<%def name="get_phase_icon(case)">
  <% phase = case.getState().getPhase() %>
  <% running = not phase%2 and 'running' or 'finished' %>
  <% phasepart = c.phases.get_phase(str(phase)) %>
  <% phasepart = c.phases.get_phase(str(phase)) %>
  %if not phasepart:
      ## use unknown phase instead of causing traceback
      <% phasepart = c.phases.get_phase("-1") %>
  %endif
  <% description = h.literal(phasepart.getDescription()) %>
  <% icon = phasepart.getIcon() %>
  <img src="${h.url(str('/images/icons/case_status/%s_%s.png' % (icon, running)) )}" alt="${description}" title="${description}" width="22" height="22">
</%def>

<%def name="render_phase_legend()">
<% legend = [] %>
<div class="container widget info">
  <strong>${_('Legend')}: </strong>
  % for p in c.phases:
    ${legend.append("%s: %s" % (g.mpuls_config.get('phases', 'shortdescription')[0][p.id], p.getDescription()))}
  % endfor
  ${h.literal(", ".join(legend))}
  | <span style="background-image:url('${h.url("/images/icons/phase-running-22.png")}'); background-repeat:no-repeat; background-position:1px -2px">
      (&nbsp;&nbsp;&nbsp;)
    </span>: ${_("Running")},
    <span style="background-image:url('${h.url("/images/icons/phase-stop-22.png")}'); background-repeat:no-repeat; background-position:1px -2px">
      (&nbsp;&nbsp;&nbsp;)
    </span>: ${_("Finished")}
</div>
<br>
</%def>

<%def name="get_sync_icon(case)">
  % if g.mpuls_config.is_enabled('case-module', 'meta-server'):
    % if case.getMeta().is_linked():
      <img src    = "${h.url('/images/icons/air-link-ok.png')}"
           border = "0"
           alt    = "${_('Case is linked with AiR-Server')}"
           title  = "${_('Case is linked with AiR-Server')}">
    % else:
      <img src    = "${h.url('/images/icons/air-link-notok.png')}"
           border = "0"
           alt    = "${_('Case is not linked with AiR-Server')}"
           title  = "${_('Case is not linked with AiR-Server')}">
    % endif
  % endif
</%def>

<%def name="get_state_icon(case)">
  % if h.hasRole(['cm']) and g.mpuls_config.is_enabled('case-module', 'organisation'):
    % if case.user_is_editor(session.get('USER_AUTHORIZED').id):
      <img src    = "${h.url('/images/icons/edit_editor.png')}"
           border = "0"
           alt    = "${_('Editor')}"
           title  = "${_('Editor')}"
           width  = "22"
           height = "22">
    % else:
      <img src    = "${h.url('/images/icons/edit_editors_22.png')}"
           border = "0"
           alt    = "${_('Standin')}"
           title  = "${_('Standin')}"
           width  = "22"
           height = "22">
    % endif
  % endif
  % if case.getState().getState() in (1,2):
    <img src    = "${h.url('/images/icons/edit_22.png')}"
         border = "0"
         alt    = "${_('Editable')}"
         title  = "${_('Editable')}"
         width  = "22"
         height = "22">
  % elif case.getState().getState() == 3:
    <img src    = "${h.url('/images/icons/delete_inactive_22.png')}"
         border = "0"
         alt    = "${_('Mark delete')}"
         title  = "${_('Mark delete')}"
         width  = "22"
         height = "22">
  % elif case.getState().getState() == 4:
    <img src    = "${h.url('/images/icons/anonymise_inactive_22.png')}"
         border = "0"
         alt    = "${_('Mark anonym')}"
         title  = "${_('Mark anonym')}"
         width  = "22"
         height = "22">
  % elif case.getState().getState() == 5:
    <img src    = "${h.url('/images/icons/anonymise_inactive_22.png')}"
         border = "0"
         alt    = "${_('Make anonymousym')}"
         title  = "${_('Make anonymousym')}"
         width  = "22"
         height = "22">
  % endif
</%def>

<%def name="get_actions(case)">
  % if h.hasRole(['admin']):
    % if case.getState().getState() != 5:
      <a href="${h.url(controller='case', action='select', id=case.id,
	     		 confirmed=0)}">
      <img src    = "${h.url('/images/icons/open_active_22.png')}"
	   border = "0"
	   alt    = "${_('Show')}"
	   title  = "${_('Show')}"
	   width  = "22"
	   height = "22"></a>
    % endif
    % if case.getState().getState() in (3, 5):
      <a href="${h.url(controller='case', action='delete', id=case.id,
                           confirmed=0)}">
        <img src    = "${h.url('/images/icons/delete_active_22.png')}"
             border = "0"
             alt    = "${_('Delete case')}"
             title  = "${_('Delete case')}"
             width  = "22"
             height = "22"></a>
    % endif
    % if case.getState().getState() == 4:
      % if g.mpuls_config.is_enabled('case-module', 'anonymize'):
        <a href="${h.url(controller='case', action='anonymize', id=case.id,
                             confirmed=0)}">
          <img src    = "${h.url('/images/icons/anonymise_active_22.png')}"
               border = "0"
               alt    = "${_('Anonymize case document')}"
               title  = "${_('Anonymize case document')}"
               width  = "22"
               height = "22"></a>
      % endif
    % endif
    % if case.getState().getState() in (3, 4):
      <a href="${h.url(controller='case', action='restore', id=case.id, 
                           confirmed=0)}">
        <img src    = "${h.url('/images/icons/refresh_active_22.png')}" 
             border = "0"
             alt    = "${_('Restore case document')}"
             title  = "${_('Restore case document')}"
             width  = "22"
             height = "22"></a>
    % endif
  % elif h.hasRole(['cm']) and case.getState().getState() in (1, 2):
    % if case.getState().getState() != 5 and  case.getState().getState() != 4:
      <a href="${h.url(controller='case', action='select', id=case.id,
	     		 confirmed=0)}">
      <img src    = "${h.url('/images/icons/open_active_22.png')}"
	   border = "0"
	   alt    = "${_('Show')}"
	   title  = "${_('Show')}"
	   width  = "22"
	   height = "22"></a>
    % endif
    % if g.mpuls_config.is_enabled('case-module', 'anonymize'):
      <a href="${h.url(controller='case', 
                           action='markForAnonymizeFromOverview', id=case.id,
                           confirmed=0)}">
        <img src    = "${h.url('/images/icons/anonymise_active_22.png')}" 
             border = "0"
             alt    = "${_('Anonymize case document')}"
             title  = "${_('Anonymize case document')}"
             width  = "22"
             height = "22"></a>
    % endif
    <a href="${h.url(controller='case', action='markForDelete', id=case.id,
                         confirmed=0)}">
      <img src    = "${h.url('/images/icons/delete_active_22.png')}" 
           border = "0"
           alt    = "${_('Delete')}"
           title  = "${_('Delete')}"
           width  = "22"
           height = "22"></a>
  % endif
</%def>

<table id="table_caseoverview" class="hovertable">
  <tr>
    ${build_list_headers()}
  </tr>
  % for num, case in enumerate(c.cases.getDatasets()):
      <tr class="${num % 2 and 'table_row_h' or ''}">
        ${build_list_row(case)}
      </tr>
  % endfor
  % if len(c.cases.getDatasets()) <= 0:
  <tr>
    <td colspan="${len(fields)+1}">
      ${_('No data found.')}
    </td>
  </tr>
  % endif
</table>
${render_phase_legend()}
