## -*- coding: utf-8 -*-
<%!

from pylons import app_globals as g
import mpulsweb.lib.helper.filters as F
import mpulsweb.lib.helpers as h
from mpulsweb.lib.translation import _

all_roles = ['admin', 'cm']

## Fields to show in the case list.  Each entry is a tuple of the form
##     (TITLE, FIELDNAME, STYLE, ROLES, FORMATTER)
## Here, TITLE is the title of the field, FIELDNAME is the field name
## used in the search URL that's built for each column, STYLE is a CSS
## class name to use in the style attribute of the HTML TH-element,
## ROLES is a list with the names of the roles for which the field
## should be shown.  Finally, FORMATTER is a python callable that is
## called with the case object as argument and which should return the
## formatted value for the column.

fields = []
for d in g.agency.get_overview_fields():
  f = d.name
  fn = d.overview_label
  fs = d.overview_style
  field = (fn,f,fs, all_roles,
           lambda case, d=d: h.shorten(h.format_object(case.get_value(d.name)), d.overview_length))
  fields.append(field)
%>

<%def name="build_list_headers()">
  % if g.mpuls_config.is_enabled('case-module', 'casebundle'):
    <th class="table_header_h table_checkbox">&nbsp;</th>
  % endif
  % for title, urlpart, style, roles, get_value in fields:
    % if h.hasRole(roles):
      <th class="${style} table_header_h">${title}
        <a href="${h.url(controller='evaluation_overview', action='overview',
                             sort_field=urlpart, sort_order='asc')}">
            <img class   = "sorticon"
            %if c.sort_field == urlpart and c.sort_order == 'asc':
                src    = "${h.url('/images/icons/sort_asc_active.png')}"
                alt    = "${_("Sorted by %s in ascending order") % title}"
                title  = "${_("Sorted by %s in ascending order") % title}"
            %else:
                src    = "${h.url('/images/icons/sort_asc.png')}"
                alt    = "${_("Sort in ascending order")}"
                title  = "${_("Sort in ascending order")}"
            %endif
                width  = "10"
                height = "9"></a>
        <a href="${h.url(controller='evaluation_overview', action='overview',
                             sort_field=urlpart, sort_order='desc')}">
            <img class   = "sorticon"
            %if c.sort_field == urlpart and c.sort_order == 'desc':
                src    = "${h.url('/images/icons/sort_desc_active.png')}"
                alt    = "${_("Sorted by %s in descending order") % title}"
                title  = "${_("Sorted by %s in descending order") % title}"
            %else:
                src    = "${h.url('/images/icons/sort_desc.png')}"
                alt    = "${_("Sort in descending order")}"
                title  = "${_("Sort in descending order")}"
            %endif
                width  = "10"
                height = "9"></a>
      </th>
    % endif
  % endfor
</%def>

<%def name="build_list_row(case)">
  % if g.mpuls_config.is_enabled('case-module', 'casebundle'):
    <td><input type="checkbox" name="case_id" value="${case.id}"></td>
  % endif
  % for title, urlpart, style, roles, get_value in fields:
    % if h.hasRole(roles):
      <td class="${style}">${get_value(case) | F.NA}</td>
    % endif
  % endfor
</%def>

<div class="agencyoverview">
<table id="table_caseoverview" class="hovertable">
  <tr>
    ${build_list_headers()}
  </tr>
  % for num, case in enumerate(c.cases.getDatasets()):
      <tr class="${num % 2 and 'table_row_h' or ''}">
        ${build_list_row(case)}
      </tr>
  % endfor
  % if len(c.cases.getDatasets()) <= 0:
  <tr>
    <td colspan="6">
      ${_('No data found.')}
    </td>
  </tr>
  % endif
</table>
</div>
