#!/bin/sh

usage()
{
cat << EOF
usage: $0 options

This script will list available databases matching the typical mpuls database naming.

OPTIONS:
   -h      Show this message
   -p      Port of the database (Default 5432)
   -v      Verbose
EOF
}

# DEFAULTS
ARG1=${1}
VERBOSE=0
DBPORT=5432

# Parse Options
while getopts "hvp:" OPTION
do
        case $OPTION in
            h)
                usage
                exit 1
                ;;
            p)
                DBPORT=$OPTARG
                ;;
            v)
                VERBOSE=1
                ;;
            ?)
                # Will be set if the optstring begins with a ":". In this case
                # all options not defined in the optstring will OPTION set to
                # "?". Otherwise the shell reports an error on the unknown
                # option.
                usage
                exit
                ;;
        esac
done
DBLIST=`psql -p ${DBPORT} -l | awk '/ka_(01jk)?[0-9]+_db/ { print $1 }' | awk -F_ '{print $2}'`
echo $DBLIST | sort
