# -*- coding: utf-8 -*-
#
# Copyright (c) 2012 Intevation GmbH
#
# libformed is free software under the terms of the
# GNU General Public License Version 3. See file
# LICENSE coming with the source of FormEd for details.
#
# Author:
# Roland Geider  <roland.geider@intevation.de>

from formed.export.sql import *

DROP_VIEW = "DROP VIEW IF EXISTS %s CASCADE;\n"

# Views from 04_views.sql
OTHER_VIEWS = ['master_tbl_eval_incm_view',
               'ka_benutzer_tbl_view',
               'nm_benutzer_master_tbl_view',
               'ka_benutzergruppe_tbl_view',
               'nm_benutzer_gruppe_tbl_view',
               'ka_konfiguration_tbl_view',
               'ka_global_dokumente_tbl_view',
               'ka_fall_dokumente_tbl_view',
               'ka_global_termine_tbl_view',
               'ka_fall_termine_tbl_view',
               'ka_status_tbl_view',
               'ka_logbuch_tbl_view',
               'ka_nachrichten_art_tbl_view',
               'ka_nachrichten_tbl_view',
               'ka_meta_tbl_view',
               'tags_view',
               'nm_tags_cases_view',
               ]

class ExportDelViews(ExportAsSQL):
    @staticmethod
    def getDescription():
        return "Export document as DROP VIEW statements"
    
    def doExport(self, path):
        root = self.root

        out = None
        creator = SchemaCreator(root)
        creator.createSchema()
        tables = creator.topological()
        arr = self.printRepeats(tables)

        try:
            out = codecs.open(path, "wb", "UTF-8")
            pfeil = out.write
            pfeil(u'--\n-- Schema generated by libformed\n-- %s\n'
                  % strftime("%a, %Y-%m-%d %H:%M:%S +0000", gmtime()))
            pfeil('-- # Revision: $Revision$\n--\n')

            pfeil('BEGIN;')
            pfeil('SET ROLE :db_adm;')
            
            


            # drop views for master table and repeat groups
            pfeil("--\n--\n-- views for master table and repeat groups\n--\n")
            for raw_table, joins, linear in arr:
                table = raw_table.name
                tab = sqlName(table) + "_view"
                pfeil(DROP_VIEW % tab)


                # anonymization view
                pfeil(DROP_VIEW % (tab + '_anonym_view'))

            
            # drop main evaluation view
            pfeil(DROP_VIEW % EVALUATION_VIEW)
            
            # Drop views from 04views.sql
            pfeil("--\n--\n-- views from 04views.sql\n--\n")
            for view in OTHER_VIEWS:
                pfeil(DROP_VIEW % view)
            pfeil('SET ROLE postgres;')
            pfeil('COMMIT;\n')
        finally:
            if out:
                try:
                    out.close()
                except:
                    pass