# -*- coding: utf-8 -*-
#
# (c) 2008 by Intevation GmbH
# This is Free software under the GPLv3.
# See LICENSE comming with the source of 'mpuls'
# for details.
#
# author: Torsten Irländer <torsten.irlaender@intevation.de>
import logging
import sys
import traceback
import psycopg2 as dbapi
import psycopg2.extensions
psycopg2.extensions.register_type(psycopg2.extensions.UNICODE)


CONFIG = {'url': None,
          'port': 5432,
          'dbname': 'waskoauswertung',
          'nameschema': '%(dbname)s',
          'userschema': '%(username)s',
          'login': 'postgres',
          'password': ''}

log = logging.getLogger(__name__)

def quote_connect_parameter(s):
    """Return a quoted version of the string s for psycopg2.connect."""
    return "'" + s.replace("\\", "\\\\").replace("'", "\\'") + "'"

class DB:

    def __init__(self, dbname, user='postgres', password='', host=None,
                 port=5432):
        self.dbname = dbname
        self.user = user
        self.password = password
        self.host = host
        self.port = int(port)
        self.conn = None
        self.connections = []

    def getConnection(self):
        for used in self.connections:
            if not used[0]:
                used[0] = True
                return used[1]
        try:
            if self.host:
                con = dbapi.connect(database=quote_connect_parameter(self.dbname),
                                    host=quote_connect_parameter(self.host),
                                    port=self.port,
                                    user=quote_connect_parameter(self.user),
                                    password=quote_connect_parameter(self.password))
            else:
                con = dbapi.connect(database=quote_connect_parameter(self.dbname),
                                    port=self.port,
                                    user=quote_connect_parameter(self.user),
                                    password=quote_connect_parameter(self.password))
            used = [True, con]
            self.connections.append(used)
            return con
        except:
            raise

    def recycleConnection(self, con, cur=None):
        if cur:
            try:
                cur.close()
            except:
                log.exception('Error on recycling connection')
        if con:
            for used in self.connections:
                if used[1] == con:
                    used[0] = False
                    return
            log.warning('Tried to recycle unused connection')

    def closeConnections(self):
        cons = self.connections
        self.connections = []
        for used in cons:
            if used[0]:
                log.warning('Closing an unrecycled connection')
            try:
                used[1].close()
            except:
                log.exception('Error on closing connection')


def get_db(username, password, dbname, host, port):
    conn, cursor = None, None
    HOST = host
    PORT = port
    DBNAME = CONFIG.get('nameschema')
    USER = CONFIG.get('userschema')
    dbname = dbname
    password = password
    username = username
    try:
        mydb = DB(dbname=DBNAME % {'dbname': dbname},
                  host=HOST, port=PORT,
                  user=USER % {'dbname': dbname, 'username': username},
                  password=password)
        return mydb
    except StandardError, err:
        log.exception('Error on get db object')
        raise


# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
